import pickle
import unittest

from troposphere import (
    AWSHelperFn,
    AWSObject,
    AWSProperty,
    Cidr,
    GenericHelperFn,
    If,
    Join,
    NoValue,
    Output,
    Parameter,
    Ref,
    Region,
    Split,
    Sub,
    Template,
    cloudformation,
    depends_on_helper,
)
from troposphere.cloudformation import Stack
from troposphere.ec2 import Instance, NetworkInterface, Route, SecurityGroupRule
from troposphere.elasticloadbalancing import HealthCheck
from troposphere.s3 import Bucket, PublicRead
from troposphere.validators import positive_integer


class TypeComparator:
    """Helper to test the __eq__ protocol"""

    def __init__(self, valid_types):
        self.valid_types = valid_types

    def __eq__(self, other):
        return isinstance(other, self.valid_types)

    def __ne__(self, other):
        return not self == other


def double(x):
    return positive_integer(x) * 2


def call_correct(x):
    return x


def call_incorrect(x):
    raise ValueError


class FakeAWSProperty(AWSProperty):
    props = {}


class FakeAWSObject(AWSObject):
    type = "Fake::AWS::Object"

    props = {
        "callcorrect": (call_correct, False),
        "callincorrect": (call_incorrect, False),
        "singlelist": (list, False),
        "multilist": ([bool, int, float], False),
        "multituple": ((bool, int), False),
        "helperfun": (positive_integer, False),
        "listhelperfun": ([double], False),
    }

    def validate(self):
        properties = self.properties
        title = self.title
        type = self.type
        if "callcorrect" in properties and "singlelist" in properties:
            raise ValueError(
                (
                    "Cannot specify both 'callcorrect and 'singlelist' in "
                    "object %s (type %s)" % (title, type)
                )
            )


class TestBasic(unittest.TestCase):
    def test___eq__(self):
        """Test __eq__."""
        assert FakeAWSObject("foobar", callcorrect=True) == FakeAWSObject(
            "foobar", callcorrect=True
        )
        assert FakeAWSObject("foobar", callcorrect=True) == {
            "title": "foobar",
            "Properties": {"callcorrect": True},
        }
        assert FakeAWSObject("foobar", callcorrect=True) == TypeComparator(AWSObject)
        assert TypeComparator(AWSObject) == FakeAWSObject("foobar", callcorrect=True)

        assert GenericHelperFn("foobar") == GenericHelperFn("foobar")
        assert GenericHelperFn({"foo": "bar"}) == {"foo": "bar"}
        assert GenericHelperFn("foobar") == TypeComparator(AWSHelperFn)
        assert TypeComparator(AWSHelperFn) == GenericHelperFn("foobar")

    def test___ne__(self):
        """Test __ne__."""
        assert FakeAWSObject("foo", callcorrect=True) != FakeAWSObject(
            "bar", callcorrect=True
        )
        assert FakeAWSObject("foobar", callcorrect=True) != FakeAWSObject(
            "foobar", callcorrect=False
        )
        assert FakeAWSObject("foobar", callcorrect=True) != FakeAWSProperty("foobar")
        assert FakeAWSObject("foobar", callcorrect=True) != TypeComparator(AWSHelperFn)
        assert TypeComparator(AWSHelperFn) != FakeAWSObject("foobar", callcorrect=True)

        assert GenericHelperFn("foobar") != GenericHelperFn("bar")
        assert GenericHelperFn("foobar") != "foobar"
        assert GenericHelperFn("foobar") != FakeAWSProperty("foobar")
        assert GenericHelperFn("foobar") != TypeComparator(AWSObject)
        assert TypeComparator(AWSObject) != GenericHelperFn("foobar")

        # Testing __ne__ for objects missing required properties
        assert Parameter("param1") != Parameter("param2")
        assert Stack("stack1") != Stack("stack2")

    def test___hash__(self):
        """Test __hash__."""
        from troposphere.ec2 import SecurityGroup

        test_sg = SecurityGroup(
            "TestSg",
            VpcId="myVPC",
            GroupDescription="Test Security Group",
        )
        _ = hash(test_sg)

    def test_badproperty(self):
        with self.assertRaises(AttributeError):
            Instance(
                "ec2instance",
                foobar=True,
            )

    def test_badrequired(self):
        with self.assertRaises(ValueError):
            t = Template()
            t.add_resource(NetworkInterface("networkinterface"))
            t.to_json()

    def test_badtype(self):
        with self.assertRaises(AttributeError):
            Instance("ec2instance", image_id=0.11)

    def test_goodrequired(self):
        NetworkInterface("interface", SubnetId="abc123")

    def test_extraattribute(self):
        class ExtendedInstance(Instance):
            def __init__(self, *args, **kwargs):
                self.attribute = None
                super().__init__(*args, **kwargs)

        instance = ExtendedInstance("ec2instance", attribute="value")
        self.assertEqual(instance.attribute, "value")

    def test_depends_on_helper_with_resource(self):
        resource_name = "Bucket1"
        b1 = Bucket(resource_name)
        self.assertEqual(depends_on_helper(b1), resource_name)

    def test_depends_on_helper_with_string(self):
        resource_name = "Bucket1"
        self.assertEqual(depends_on_helper(resource_name), resource_name)

    def test_resource_depends_on(self):
        b1 = Bucket("B1")
        b2 = Bucket("B2", DependsOn=b1)
        self.assertEqual(b1.title, b2.resource["DependsOn"])

    def test_resource_depends_on_attr(self):
        b1 = Bucket("B1")
        b2 = Bucket("B2", DependsOn=b1)
        self.assertEqual(b1.title, b2.DependsOn)

    def test_resource_depends_on_list(self):
        b1 = Bucket("B1")
        b2 = Bucket("B2")
        b3 = Bucket("B3", DependsOn=[b1, b2])
        self.assertEqual(b1.title, b3.DependsOn[0])
        self.assertEqual(b2.title, b3.DependsOn[1])

    def test_pickle_ok(self):
        # tests that objects can be pickled/un-pickled without hitting issues
        bucket_name = "test-bucket"
        b = Bucket("B1", BucketName=bucket_name)
        p = pickle.dumps(b)
        b2 = pickle.loads(p)
        self.assertEqual(b2.BucketName, b.BucketName)


class TestValidators(unittest.TestCase):
    def test_callcorrect(self):
        FakeAWSObject("fake", callcorrect=True)

    def test_callincorrect(self):
        with self.assertRaises(ValueError):
            FakeAWSObject("fake", callincorrect=True)

    def test_list(self):
        FakeAWSObject("fake", singlelist=["a", 1])

    def test_badlist(self):
        with self.assertRaises(TypeError):
            FakeAWSObject("fake", singlelist=True)

    def test_multilist(self):
        FakeAWSObject("fake", multilist=[1, True, 2, 0.3])

    def test_badmultilist(self):
        with self.assertRaises(TypeError):
            FakeAWSObject("fake", multilist=True)
        with self.assertRaises(TypeError):
            FakeAWSObject("fake", multilist=[1, "a"])

    def test_mutualexclusion(self):
        t = Template()
        t.add_resource(FakeAWSObject("fake", callcorrect=True, singlelist=[10]))
        with self.assertRaises(ValueError):
            t.to_json()

    def test_tuples(self):
        FakeAWSObject("fake", multituple=True)
        FakeAWSObject("fake", multituple=10)
        with self.assertRaises(TypeError):
            FakeAWSObject("fake", multituple=0.1)

    def test_helperfun(self):
        FakeAWSObject("fake", helperfun=Ref("fake_ref"))

    def test_listhelperfun(self):
        with self.assertRaises(TypeError):
            FakeAWSObject("fake", listhelperfun=1)

        x = FakeAWSObject("fake", listhelperfun=[1, 2])
        if x.listhelperfun != [2, 4]:
            raise ValueError

        with self.assertRaises(ValueError):
            FakeAWSObject("fake", listhelperfun=[1, -2])

        with self.assertRaises(ValueError):
            FakeAWSObject("fake", listhelperfun=[1, "foo"])

    def test_exception(self):
        def ExceptionValidator(x):
            raise ValueError

        class ExceptionAWSProperty(AWSProperty):
            props = {
                "foo": (ExceptionValidator, True),
            }

        with self.assertRaises(ValueError):
            ExceptionAWSProperty(foo="bar")

    def test_error_message(self):
        from troposphere.cloudfront import CustomHeader

        with self.assertRaisesRegex(ValueError, "troposphere.cloudfront.CustomHeader"):
            CustomHeader(Header="Cache-Policy", Value="no-cache").to_dict()


class TestHealthCheck(unittest.TestCase):
    def test_healthy_interval_ok(self):
        HealthCheck(
            HealthyThreshold="2",
            Interval="2",
            Target="HTTP:80/index.html",
            Timeout="4",
            UnhealthyThreshold="9",
        )

    def test_healthy_interval_too_low(self):
        with self.assertRaises(ValueError):
            HealthCheck(
                HealthyThreshold="1",
                Interval="2",
                Target="HTTP:80/index.html",
                Timeout="4",
                UnhealthyThreshold="9",
            )


class TestOutput(unittest.TestCase):
    def test_noproperty(self):
        t = Output("MyOutput", Value="myvalue")
        d = t.to_dict()
        with self.assertRaises(KeyError):
            d["Properties"]

    def test_empty_awsproperty_outputs_empty_object(self):
        t = FakeAWSProperty()
        d = t.to_dict()
        self.assertEqual(len(d), 0)


class TestParameter(unittest.TestCase):
    def test_noproperty(self):
        t = Parameter("MyParameter", Type="String")
        d = t.to_dict()
        with self.assertRaises(KeyError):
            d["Properties"]

    def test_property_validator(self):
        p = Parameter("BasicString", Type="String", MaxLength=10)
        p.validate()

        p = Parameter("BasicString", Type="String", MaxValue=10)
        with self.assertRaises(ValueError):
            p.validate()

        p = Parameter("BasicNumber", Type="Number", MaxValue=10)
        p.validate()

        p = Parameter("BasicNumber", Type="Number", AllowedPattern=".*")
        with self.assertRaises(ValueError):
            p.validate()

    def test_invalid_parameter_property_in_template(self):
        t = Template()
        p = Parameter("BasicNumber", Type="Number", AllowedPattern=".*")
        t.add_parameter(p)
        with self.assertRaises(ValueError):
            t.to_json()

    def test_get_or_add_adds(self):
        t = Template()
        p = Parameter("param", Type="String", Default="foo")
        result = t.get_or_add_parameter(p)
        self.assertEqual(t.parameters["param"], p)
        self.assertEqual(result, p)

    def test_add_or_get_returns_with_out_adding_duplicate(self):
        t = Template()
        p = Parameter("param", Type="String", Default="foo")
        t.add_parameter(p)
        result = t.get_or_add_parameter(p)
        self.assertEqual(t.parameters["param"], p)
        self.assertEqual(result, p)
        self.assertEqual(len(t.parameters), 1)

    def test_property_default(self):
        p = Parameter("param", Type="String", Default="foo")
        p.validate()

        p = Parameter("param", Type="Number", Default=1)
        p.validate()

        p = Parameter("param", Type="Number", Default=1.0)
        p.validate()

        p = Parameter("param", Type="Number", Default=0.1)
        p.validate()

        p = Parameter("param", Type="List<Number>", Default="1, 2, 3")
        p.validate()

        p = Parameter("param", Type="List<Number>", Default=" 0.1 , 2 , 1.1 ")
        p.validate()

        with self.assertRaises(ValueError):
            p = Parameter("param", Type="String", Default=1)
            p.validate()

        with self.assertRaises(ValueError):
            p = Parameter("param", Type="Number", Default="foo")
            p.validate()

        with self.assertRaises(TypeError):
            p = Parameter("param", Type="Number", Default=["foo"])
            p.validate()

        with self.assertRaises(ValueError):
            p = Parameter("param", Type="List<Number>", Default="foo")
            p.validate()

        with self.assertRaises(ValueError):
            p = Parameter("param", Type="List<Number>", Default="1, 2, foo")
            p.validate()

        with self.assertRaises(TypeError):
            p = Parameter("param", Type="List<Number>", Default=["1", "2"])
            p.validate()


class TestProperty(unittest.TestCase):
    def test_noproperty(self):
        t = SecurityGroupRule(
            IpProtocol="tcp",
            FromPort="22",
            ToPort="22",
            CidrIp="0.0.0.0/0",
        )
        d = t.to_dict()
        with self.assertRaises(KeyError):
            d["Properties"]

    def test_awsproperty_invalid_property(self):
        t = FakeAWSProperty()
        with self.assertRaises(AttributeError) as context:
            t.badproperty = 5
        self.assertTrue("FakeAWSProperty" in context.exception.args[0])
        self.assertTrue("badproperty" in context.exception.args[0])


class TestDuplicate(unittest.TestCase):
    def test_output(self):
        t = Template()
        o = Output("MyOutput", Value="myvalue")
        t.add_output(o)
        with self.assertRaises(ValueError):
            t.add_output(o)

    def test_parameter(self):
        t = Template()
        p = Parameter("MyParameter", Type="String")
        t.add_parameter(p)
        with self.assertRaises(ValueError):
            t.add_parameter(p)

    def test_resource(self):
        t = Template()
        r = FakeAWSObject("fake", callcorrect=True)
        t.add_resource(r)
        with self.assertRaises(ValueError):
            t.add_resource(r)


class TestRef(unittest.TestCase):
    def test_ref(self):
        param = Parameter("param", Description="description", Type="String")
        t = Ref(param)
        ref = t.to_dict()
        self.assertEqual(ref["Ref"], "param")

    def test_ref_eq(self):
        s = "AWS::NoValue"
        r = Ref(s)

        wch = cloudformation.WaitConditionHandle("TestResource")

        self.assertEqual(s, r)
        self.assertEqual(s, NoValue)
        self.assertEqual(r, NoValue)
        self.assertEqual(wch.Ref(), "TestResource")

        self.assertNotEqual(r, "AWS::Region")
        self.assertNotEqual(r, Region)
        self.assertNotEqual(r, Ref)
        self.assertNotEqual(wch.Ref(), "NonexistantResource")

    def test_ref_hash(self):
        s = hash("AWS::NoValue")
        r = hash(Ref("AWS::NoValue"))

        wch = cloudformation.WaitConditionHandle("TestResource")

        self.assertEqual(s, r)
        self.assertEqual(s, hash(NoValue))
        self.assertEqual(r, hash(NoValue))
        self.assertEqual(hash(wch.Ref()), hash("TestResource"))

        self.assertNotEqual(r, hash("AWS::Region"))
        self.assertNotEqual(r, hash(Region))
        self.assertNotEqual(r, hash(Ref))
        self.assertNotEqual(hash(wch.Ref()), hash("NonexistantResource"))


class TestName(unittest.TestCase):
    def test_ref(self):
        name = "fake"
        t = Template()
        resource = t.add_resource(Instance(name))
        self.assertEqual(resource.name, name)


class TestCidr(unittest.TestCase):
    def test_getcidr(self):
        raw = Cidr("10.1.10.1/24", 2)
        actual = raw.to_dict()
        expected = {"Fn::Cidr": ["10.1.10.1/24", 2]}
        self.assertEqual(expected, actual)

    def test_getcidr_withsizemask(self):
        raw = Cidr("10.1.10.1/24", 2, 10)
        actual = raw.to_dict()
        expected = {"Fn::Cidr": ["10.1.10.1/24", 2, 10]}
        self.assertEqual(expected, actual)


class TestSub(unittest.TestCase):
    def test_sub_without_vars(self):
        s = "foo ${AWS::Region}"
        raw = Sub(s)
        actual = raw.to_dict()
        expected = {"Fn::Sub": "foo ${AWS::Region}"}
        self.assertEqual(expected, actual)

    def test_sub_with_vars_unpakaged(self):
        s = "foo ${AWS::Region} ${sub1} ${sub2}"
        values = {"sub1": "uno", "sub2": "dos"}
        raw = Sub(s, **values)
        actual = raw.to_dict()
        expected = {"Fn::Sub": ["foo ${AWS::Region} ${sub1} ${sub2}", values]}
        self.assertEqual(expected, actual)

    def test_sub_with_vars_not_unpakaged(self):
        s = "foo ${AWS::Region} ${sub1} ${sub2}"
        values = {"sub1": "uno", "sub2": "dos"}
        raw = Sub(s, values)
        actual = raw.to_dict()
        expected = {"Fn::Sub": ["foo ${AWS::Region} ${sub1} ${sub2}", values]}
        self.assertEqual(expected, actual)

    def test_sub_with_vars_mix(self):
        s = "foo ${AWS::Region} ${sub1} ${sub2} ${sub3}"
        values = {"sub1": "uno", "sub2": "dos"}
        raw = Sub(s, values, sub3="tres")
        actual = raw.to_dict()
        expected = {
            "Fn::Sub": [
                "foo ${AWS::Region} ${sub1} ${sub2} ${sub3}",
                {"sub1": "uno", "sub2": "dos", "sub3": "tres"},
            ]
        }
        self.assertEqual(expected, actual)


class TestSplit(unittest.TestCase):
    def test_split(self):
        delimiter = ","
        source_string = (
            '{ "Fn::ImportValue": { "Fn::Sub": ' '"${VpcStack}-PublicSubnets" }'
        )
        raw = Split(delimiter, source_string)
        actual = raw.to_dict()
        expected = {
            "Fn::Split": [
                ",",
                '{ "Fn::ImportValue": { ' '"Fn::Sub": "${VpcStack}-PublicSubnets" }',
            ]
        }
        self.assertEqual(expected, actual)

        with self.assertRaises(ValueError):
            Split(10, "foobar")


class TestJoin(unittest.TestCase):
    def test_join(self):
        delimiter = ","
        source_string = (
            '{ [ "arn:aws:lambda:",{ "Ref": "AWS::Region" },":",'
            '{ "Ref": "AWS::AccountId" },'
            '":function:cfnRedisEndpointLookup" ] }'
        )
        raw = Join(delimiter, source_string)
        actual = raw.to_dict()
        expected = {
            "Fn::Join": [
                ",",
                '{ [ "arn:aws:lambda:",{ "Ref": '
                '"AWS::Region" },":",{ "Ref": "AWS::AccountId" },'
                '":function:cfnRedisEndpointLookup" ] }',
            ]
        }
        self.assertEqual(expected, actual)

        with self.assertRaises(ValueError):
            Join(10, "foobar")


class TestValidation(unittest.TestCase):
    def test_validation(self):
        route = Route(
            "Route66",
            DestinationCidrBlock="0.0.0.0/0",
            RouteTableId=Ref("RouteTable66"),
            InstanceId=If("UseNat", Ref("AWS::NoValue"), Ref("UseNat")),
            NatGatewayId=If("UseNat", Ref("UseNat"), Ref("AWS::NoValue")),
        )
        t = Template()
        t.add_resource(route)
        with self.assertRaises(ValueError):
            t.to_json()

    def test_novalidation(self):
        route = Route(
            "Route66",
            validation=False,
            DestinationCidrBlock="0.0.0.0/0",
            RouteTableId=Ref("RouteTable66"),
            InstanceId=If("UseNat", Ref("AWS::NoValue"), Ref("UseNat")),
            NatGatewayId=If("UseNat", Ref("UseNat"), Ref("AWS::NoValue")),
        )
        t = Template()
        t.add_resource(route)
        t.to_json()

    def test_no_validation_method(self):
        route = Route(
            "Route66",
            DestinationCidrBlock="0.0.0.0/0",
            RouteTableId=Ref("RouteTable66"),
            InstanceId=If("UseNat", Ref("AWS::NoValue"), Ref("UseNat")),
            NatGatewayId=If("UseNat", Ref("UseNat"), Ref("AWS::NoValue")),
        ).no_validation()
        t = Template()
        t.add_resource(route)
        t.to_json()


test_updatereplacepolicy_yaml = """\
Resources:
  S3Bucket:
    Properties:
      AccessControl: PublicRead
    Type: AWS::S3::Bucket
    UpdateReplacePolicy: Retain
"""


class TestAttributes(unittest.TestCase):
    def test_BogusAttribute(self):
        t = Template()
        with self.assertRaises(AttributeError):
            t.add_resource(Bucket("S3Bucket", Bogus="Retain"))

    def test_UpdateReplacePolicy(self):
        t = Template()
        t.add_resource(
            Bucket(
                "S3Bucket",
                AccessControl=PublicRead,
                UpdateReplacePolicy="Retain",
            )
        )
        t.to_yaml()
        self.assertEqual(t.to_yaml(), test_updatereplacepolicy_yaml)


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.


def validate_growth_type(growth_type):
    """
    Property: DeploymentStrategy.GrowthType
    """

    VALID_GROWTH_TYPES = ("LINEAR",)

    if growth_type not in VALID_GROWTH_TYPES:
        raise ValueError(
            "DeploymentStrategy GrowthType must be one of: %s"
            % ", ".join(VALID_GROWTH_TYPES)
        )
    return growth_type


def validate_replicate_to(replicate_to):
    """
    Property: DeploymentStrategy.ReplicateTo
    """

    VALID_REPLICATION_DESTINATION = ("NONE", "SSM_DOCUMENT")

    if replicate_to not in VALID_REPLICATION_DESTINATION:
        raise ValueError(
            "DeploymentStrategy ReplicateTo must be one of: %s"
            % ", ".join(VALID_REPLICATION_DESTINATION)
        )
    return replicate_to


def validate_validator_type(validator_type):
    """
    Property: Validators.Type
    """

    VALID_VALIDATOR_TYPE = ("JSON_SCHEMA", "LAMBDA")

    if validator_type not in VALID_VALIDATOR_TYPE:
        raise ValueError(
            "ConfigurationProfile Validator Type must be one of: %s"
            % ", ".join(VALID_VALIDATOR_TYPE)  # NOQA
        )
    return validator_type



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType, Tags
from .validators import boolean, double, integer
from .validators.apigateway import (
    dict_or_string,
    validate_authorizer_ttl,
    validate_gateway_response_type,
    validate_model,
    validate_tags_or_list,
    validate_timeout_in_millis,
)


class Account(AWSObject):
    """
    `Account <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-account.html>`__
    """

    resource_type = "AWS::ApiGateway::Account"

    props: PropsDictType = {
        "CloudWatchRoleArn": (str, False),
    }


class StageKey(AWSProperty):
    """
    `StageKey <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-apikey-stagekey.html>`__
    """

    props: PropsDictType = {
        "RestApiId": (str, False),
        "StageName": (str, False),
    }


class ApiKey(AWSObject):
    """
    `ApiKey <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-apikey.html>`__
    """

    resource_type = "AWS::ApiGateway::ApiKey"

    props: PropsDictType = {
        "CustomerId": (str, False),
        "Description": (str, False),
        "Enabled": (boolean, False),
        "GenerateDistinctId": (boolean, False),
        "Name": (str, False),
        "StageKeys": ([StageKey], False),
        "Tags": (Tags, False),
        "Value": (str, False),
    }


class Authorizer(AWSObject):
    """
    `Authorizer <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-authorizer.html>`__
    """

    resource_type = "AWS::ApiGateway::Authorizer"

    props: PropsDictType = {
        "AuthType": (str, False),
        "AuthorizerCredentials": (str, False),
        "AuthorizerResultTtlInSeconds": (validate_authorizer_ttl, False),
        "AuthorizerUri": (str, False),
        "IdentitySource": (str, False),
        "IdentityValidationExpression": (str, False),
        "Name": (str, True),
        "ProviderARNs": ([str], False),
        "RestApiId": (str, True),
        "Type": (str, True),
    }


class BasePathMapping(AWSObject):
    """
    `BasePathMapping <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-basepathmapping.html>`__
    """

    resource_type = "AWS::ApiGateway::BasePathMapping"

    props: PropsDictType = {
        "BasePath": (str, False),
        "DomainName": (str, True),
        "Id": (str, False),
        "RestApiId": (str, False),
        "Stage": (str, False),
    }


class ClientCertificate(AWSObject):
    """
    `ClientCertificate <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-clientcertificate.html>`__
    """

    resource_type = "AWS::ApiGateway::ClientCertificate"

    props: PropsDictType = {
        "Description": (str, False),
        "Tags": (Tags, False),
    }


class DeploymentCanarySettings(AWSProperty):
    """
    `DeploymentCanarySettings <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-deployment-deploymentcanarysettings.html>`__
    """

    props: PropsDictType = {
        "PercentTraffic": (double, False),
        "StageVariableOverrides": (dict, False),
        "UseStageCache": (boolean, False),
    }


class AccessLogSetting(AWSProperty):
    """
    `AccessLogSetting <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-stage-accesslogsetting.html>`__
    """

    props: PropsDictType = {
        "DestinationArn": (str, False),
        "Format": (str, False),
    }


class MethodSetting(AWSProperty):
    """
    `MethodSetting <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-stage-methodsetting.html>`__
    """

    props: PropsDictType = {
        "CacheDataEncrypted": (boolean, False),
        "CacheTtlInSeconds": (integer, False),
        "CachingEnabled": (boolean, False),
        "DataTraceEnabled": (boolean, False),
        "HttpMethod": (str, False),
        "LoggingLevel": (str, False),
        "MetricsEnabled": (boolean, False),
        "ResourcePath": (str, False),
        "ThrottlingBurstLimit": (integer, False),
        "ThrottlingRateLimit": (double, False),
    }


class StageDescription(AWSProperty):
    """
    `StageDescription <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-deployment-stagedescription.html>`__
    """

    props: PropsDictType = {
        "AccessLogSetting": (AccessLogSetting, False),
        "CacheClusterEnabled": (boolean, False),
        "CacheClusterSize": (str, False),
        "CacheDataEncrypted": (boolean, False),
        "CacheTtlInSeconds": (integer, False),
        "CachingEnabled": (boolean, False),
        "CanarySetting": (DeploymentCanarySettings, False),
        "ClientCertificateId": (str, False),
        "DataTraceEnabled": (boolean, False),
        "Description": (str, False),
        "DocumentationVersion": (str, False),
        "LoggingLevel": (str, False),
        "MethodSettings": ([MethodSetting], False),
        "MetricsEnabled": (boolean, False),
        "Tags": (validate_tags_or_list, False),
        "ThrottlingBurstLimit": (integer, False),
        "ThrottlingRateLimit": (double, False),
        "TracingEnabled": (boolean, False),
        "Variables": (dict, False),
    }


class Deployment(AWSObject):
    """
    `Deployment <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-deployment.html>`__
    """

    resource_type = "AWS::ApiGateway::Deployment"

    props: PropsDictType = {
        "DeploymentCanarySettings": (DeploymentCanarySettings, False),
        "Description": (str, False),
        "RestApiId": (str, True),
        "StageDescription": (StageDescription, False),
        "StageName": (str, False),
    }


class Location(AWSProperty):
    """
    `Location <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-documentationpart-location.html>`__
    """

    props: PropsDictType = {
        "Method": (str, False),
        "Name": (str, False),
        "Path": (str, False),
        "StatusCode": (str, False),
        "Type": (str, False),
    }


class DocumentationPart(AWSObject):
    """
    `DocumentationPart <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-documentationpart.html>`__
    """

    resource_type = "AWS::ApiGateway::DocumentationPart"

    props: PropsDictType = {
        "Location": (Location, True),
        "Properties": (str, True),
        "RestApiId": (str, True),
    }


class DocumentationVersion(AWSObject):
    """
    `DocumentationVersion <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-documentationversion.html>`__
    """

    resource_type = "AWS::ApiGateway::DocumentationVersion"

    props: PropsDictType = {
        "Description": (str, False),
        "DocumentationVersion": (str, True),
        "RestApiId": (str, True),
    }


class EndpointConfiguration(AWSProperty):
    """
    `EndpointConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-restapi-endpointconfiguration.html>`__
    """

    props: PropsDictType = {
        "Types": ([str], False),
        "VpcEndpointIds": ([str], False),
    }


class MutualTlsAuthentication(AWSProperty):
    """
    `MutualTlsAuthentication <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-domainname-mutualtlsauthentication.html>`__
    """

    props: PropsDictType = {
        "TruststoreUri": (str, False),
        "TruststoreVersion": (str, False),
    }


class DomainName(AWSObject):
    """
    `DomainName <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-domainname.html>`__
    """

    resource_type = "AWS::ApiGateway::DomainName"

    props: PropsDictType = {
        "CertificateArn": (str, False),
        "DomainName": (str, False),
        "EndpointConfiguration": (EndpointConfiguration, False),
        "MutualTlsAuthentication": (MutualTlsAuthentication, False),
        "OwnershipVerificationCertificateArn": (str, False),
        "RegionalCertificateArn": (str, False),
        "SecurityPolicy": (str, False),
        "Tags": (Tags, False),
    }


class GatewayResponse(AWSObject):
    """
    `GatewayResponse <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-gatewayresponse.html>`__
    """

    resource_type = "AWS::ApiGateway::GatewayResponse"

    props: PropsDictType = {
        "ResponseParameters": (dict, False),
        "ResponseTemplates": (dict, False),
        "ResponseType": (validate_gateway_response_type, True),
        "RestApiId": (str, True),
        "StatusCode": (str, False),
    }


class IntegrationResponse(AWSProperty):
    """
    `IntegrationResponse <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-method-integrationresponse.html>`__
    """

    props: PropsDictType = {
        "ContentHandling": (str, False),
        "ResponseParameters": (dict, False),
        "ResponseTemplates": (dict, False),
        "SelectionPattern": (str, False),
        "StatusCode": (str, True),
    }


class Integration(AWSProperty):
    """
    `Integration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-method-integration.html>`__
    """

    props: PropsDictType = {
        "CacheKeyParameters": ([str], False),
        "CacheNamespace": (str, False),
        "ConnectionId": (str, False),
        "ConnectionType": (str, False),
        "ContentHandling": (str, False),
        "Credentials": (str, False),
        "IntegrationHttpMethod": (str, False),
        "IntegrationResponses": ([IntegrationResponse], False),
        "PassthroughBehavior": (str, False),
        "RequestParameters": (dict, False),
        "RequestTemplates": (dict, False),
        "TimeoutInMillis": (validate_timeout_in_millis, False),
        "Type": (str, True),
        "Uri": (str, False),
    }


class MethodResponse(AWSProperty):
    """
    `MethodResponse <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-method-methodresponse.html>`__
    """

    props: PropsDictType = {
        "ResponseModels": (dict, False),
        "ResponseParameters": (dict, False),
        "StatusCode": (str, True),
    }


class Method(AWSObject):
    """
    `Method <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-method.html>`__
    """

    resource_type = "AWS::ApiGateway::Method"

    props: PropsDictType = {
        "ApiKeyRequired": (boolean, False),
        "AuthorizationScopes": ([str], False),
        "AuthorizationType": (str, False),
        "AuthorizerId": (str, False),
        "HttpMethod": (str, True),
        "Integration": (Integration, False),
        "MethodResponses": ([MethodResponse], False),
        "OperationName": (str, False),
        "RequestModels": (dict, False),
        "RequestParameters": (dict, False),
        "RequestValidatorId": (str, False),
        "ResourceId": (str, True),
        "RestApiId": (str, True),
    }


class Model(AWSObject):
    """
    `Model <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-model.html>`__
    """

    resource_type = "AWS::ApiGateway::Model"

    props: PropsDictType = {
        "ContentType": (str, False),
        "Description": (str, False),
        "Name": (str, False),
        "RestApiId": (str, True),
        "Schema": (dict_or_string, False),
    }

    def validate(self):
        validate_model(self)


class RequestValidator(AWSObject):
    """
    `RequestValidator <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-requestvalidator.html>`__
    """

    resource_type = "AWS::ApiGateway::RequestValidator"

    props: PropsDictType = {
        "Name": (str, False),
        "RestApiId": (str, True),
        "ValidateRequestBody": (boolean, False),
        "ValidateRequestParameters": (boolean, False),
    }


class Resource(AWSObject):
    """
    `Resource <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-resource.html>`__
    """

    resource_type = "AWS::ApiGateway::Resource"

    props: PropsDictType = {
        "ParentId": (str, True),
        "PathPart": (str, True),
        "RestApiId": (str, True),
    }


class S3Location(AWSProperty):
    """
    `S3Location <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-restapi-s3location.html>`__
    """

    props: PropsDictType = {
        "Bucket": (str, False),
        "ETag": (str, False),
        "Key": (str, False),
        "Version": (str, False),
    }


class RestApi(AWSObject):
    """
    `RestApi <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-restapi.html>`__
    """

    resource_type = "AWS::ApiGateway::RestApi"

    props: PropsDictType = {
        "ApiKeySourceType": (str, False),
        "BinaryMediaTypes": ([str], False),
        "Body": (dict, False),
        "BodyS3Location": (S3Location, False),
        "CloneFrom": (str, False),
        "Description": (str, False),
        "DisableExecuteApiEndpoint": (boolean, False),
        "EndpointConfiguration": (EndpointConfiguration, False),
        "FailOnWarnings": (boolean, False),
        "MinimumCompressionSize": (integer, False),
        "Mode": (str, False),
        "Name": (str, False),
        "Parameters": (dict, False),
        "Policy": (dict, False),
        "Tags": (Tags, False),
    }


class StageCanarySetting(AWSProperty):
    """
    `StageCanarySetting <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-stage-canarysetting.html>`__
    """

    props: PropsDictType = {
        "DeploymentId": (str, False),
        "PercentTraffic": (double, False),
        "StageVariableOverrides": (dict, False),
        "UseStageCache": (boolean, False),
    }


class Stage(AWSObject):
    """
    `Stage <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-stage.html>`__
    """

    resource_type = "AWS::ApiGateway::Stage"

    props: PropsDictType = {
        "AccessLogSetting": (AccessLogSetting, False),
        "CacheClusterEnabled": (boolean, False),
        "CacheClusterSize": (str, False),
        "CanarySetting": (StageCanarySetting, False),
        "ClientCertificateId": (str, False),
        "DeploymentId": (str, False),
        "Description": (str, False),
        "DocumentationVersion": (str, False),
        "MethodSettings": ([MethodSetting], False),
        "RestApiId": (str, True),
        "StageName": (str, False),
        "Tags": (validate_tags_or_list, False),
        "TracingEnabled": (boolean, False),
        "Variables": (dict, False),
    }


class ThrottleSettings(AWSProperty):
    """
    `ThrottleSettings <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-usageplan-throttlesettings.html>`__
    """

    props: PropsDictType = {
        "BurstLimit": (integer, False),
        "RateLimit": (double, False),
    }


class ApiStage(AWSProperty):
    """
    `ApiStage <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-usageplan-apistage.html>`__
    """

    props: PropsDictType = {
        "ApiId": (str, False),
        "Stage": (str, False),
        "Throttle": (dict, False),
    }


class QuotaSettings(AWSProperty):
    """
    `QuotaSettings <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-usageplan-quotasettings.html>`__
    """

    props: PropsDictType = {
        "Limit": (integer, False),
        "Offset": (integer, False),
        "Period": (str, False),
    }


class UsagePlan(AWSObject):
    """
    `UsagePlan <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-usageplan.html>`__
    """

    resource_type = "AWS::ApiGateway::UsagePlan"

    props: PropsDictType = {
        "ApiStages": ([ApiStage], False),
        "Description": (str, False),
        "Quota": (QuotaSettings, False),
        "Tags": (Tags, False),
        "Throttle": (ThrottleSettings, False),
        "UsagePlanName": (str, False),
    }


class UsagePlanKey(AWSObject):
    """
    `UsagePlanKey <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-usageplankey.html>`__
    """

    resource_type = "AWS::ApiGateway::UsagePlanKey"

    props: PropsDictType = {
        "KeyId": (str, True),
        "KeyType": (str, True),
        "UsagePlanId": (str, True),
    }


class VpcLink(AWSObject):
    """
    `VpcLink <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-apigateway-vpclink.html>`__
    """

    resource_type = "AWS::ApiGateway::VpcLink"

    props: PropsDictType = {
        "Description": (str, False),
        "Name": (str, True),
        "Tags": (Tags, False),
        "TargetArns": ([str], True),
    }


class CanarySetting(AWSProperty):
    """
    `CanarySetting <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-apigateway-deployment-canarysetting.html>`__
    """

    props: PropsDictType = {
        "PercentTraffic": (double, False),
        "StageVariableOverrides": (dict, False),
        "UseStageCache": (boolean, False),
    }



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType
from .validators import boolean, double, integer
from .validators.iottwinmaker import validate_listvalue, validate_nestedtypel


class CompositeComponentType(AWSProperty):
    """
    `CompositeComponentType <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iottwinmaker-componenttype-compositecomponenttype.html>`__
    """

    props: PropsDictType = {
        "ComponentTypeId": (str, False),
    }


class LambdaFunction(AWSProperty):
    """
    `LambdaFunction <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iottwinmaker-componenttype-lambdafunction.html>`__
    """

    props: PropsDictType = {
        "Arn": (str, True),
    }


class DataConnector(AWSProperty):
    """
    `DataConnector <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iottwinmaker-componenttype-dataconnector.html>`__
    """

    props: PropsDictType = {
        "IsNative": (boolean, False),
        "Lambda": (LambdaFunction, False),
    }


class Function(AWSProperty):
    """
    `Function <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iottwinmaker-componenttype-function.html>`__
    """

    props: PropsDictType = {
        "ImplementedBy": (DataConnector, False),
        "RequiredProperties": ([str], False),
        "Scope": (str, False),
    }


class RelationshipValue(AWSProperty):
    """
    `RelationshipValue <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iottwinmaker-entity-relationshipvalue.html>`__
    """

    props: PropsDictType = {
        "TargetComponentName": (str, False),
        "TargetEntityId": (str, False),
    }


class DataValue(AWSProperty):
    """
    `DataValue <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iottwinmaker-entity-datavalue.html>`__
    """

    props: PropsDictType = {
        "BooleanValue": (boolean, False),
        "DoubleValue": (double, False),
        "Expression": (str, False),
        "IntegerValue": (integer, False),
        "ListValue": (validate_listvalue, False),
        "LongValue": (double, False),
        "MapValue": (dict, False),
        "RelationshipValue": (RelationshipValue, False),
        "StringValue": (str, False),
    }


class Relationship(AWSProperty):
    """
    `Relationship <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iottwinmaker-entity-relationship.html>`__
    """

    props: PropsDictType = {
        "RelationshipType": (str, False),
        "TargetComponentTypeId": (str, False),
    }


class DataType(AWSProperty):
    """
    `DataType <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iottwinmaker-entity-datatype.html>`__
    """

    props: PropsDictType = {
        "AllowedValues": ([DataValue], False),
        "NestedType": (validate_nestedtypel, False),
        "Relationship": (Relationship, False),
        "Type": (str, False),
        "UnitOfMeasure": (str, False),
    }


class PropertyDefinition(AWSProperty):
    """
    `PropertyDefinition <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iottwinmaker-componenttype-propertydefinition.html>`__
    """

    props: PropsDictType = {
        "Configurations": (dict, False),
        "DataType": (DataType, False),
        "DefaultValue": (DataValue, False),
        "IsExternalId": (boolean, False),
        "IsRequiredInEntity": (boolean, False),
        "IsStoredExternally": (boolean, False),
        "IsTimeSeries": (boolean, False),
    }


class PropertyGroup(AWSProperty):
    """
    `PropertyGroup <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iottwinmaker-entity-propertygroup.html>`__
    """

    props: PropsDictType = {
        "GroupType": (str, False),
        "PropertyNames": ([str], False),
    }


class ComponentType(AWSObject):
    """
    `ComponentType <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iottwinmaker-componenttype.html>`__
    """

    resource_type = "AWS::IoTTwinMaker::ComponentType"

    props: PropsDictType = {
        "ComponentTypeId": (str, True),
        "CompositeComponentTypes": (dict, False),
        "Description": (str, False),
        "ExtendsFrom": ([str], False),
        "Functions": (dict, False),
        "IsSingleton": (boolean, False),
        "PropertyDefinitions": (dict, False),
        "PropertyGroups": (dict, False),
        "Tags": (dict, False),
        "WorkspaceId": (str, True),
    }


class Definition(AWSProperty):
    """
    `Definition <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iottwinmaker-entity-definition.html>`__
    """

    props: PropsDictType = {
        "Configuration": (dict, False),
        "DataType": (DataType, False),
        "DefaultValue": (DataValue, False),
        "IsExternalId": (boolean, False),
        "IsFinal": (boolean, False),
        "IsImported": (boolean, False),
        "IsInherited": (boolean, False),
        "IsRequiredInEntity": (boolean, False),
        "IsStoredExternally": (boolean, False),
        "IsTimeSeries": (boolean, False),
    }


class Property(AWSProperty):
    """
    `Property <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iottwinmaker-entity-property.html>`__
    """

    props: PropsDictType = {
        "Definition": (Definition, False),
        "Value": (DataValue, False),
    }


class Error(AWSProperty):
    """
    `Error <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iottwinmaker-entity-error.html>`__
    """

    props: PropsDictType = {
        "Code": (str, False),
        "Message": (str, False),
    }


class Status(AWSProperty):
    """
    `Status <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iottwinmaker-entity-status.html>`__
    """

    props: PropsDictType = {
        "Error": (Error, False),
        "State": (str, False),
    }


class Component(AWSProperty):
    """
    `Component <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iottwinmaker-entity-component.html>`__
    """

    props: PropsDictType = {
        "ComponentName": (str, False),
        "ComponentTypeId": (str, False),
        "DefinedIn": (str, False),
        "Description": (str, False),
        "Properties": (dict, False),
        "PropertyGroups": (dict, False),
        "Status": (Status, False),
    }


class CompositeComponent(AWSProperty):
    """
    `CompositeComponent <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-iottwinmaker-entity-compositecomponent.html>`__
    """

    props: PropsDictType = {
        "ComponentName": (str, False),
        "ComponentPath": (str, False),
        "ComponentTypeId": (str, False),
        "Description": (str, False),
        "Properties": (dict, False),
        "PropertyGroups": (dict, False),
        "Status": (Status, False),
    }


class Entity(AWSObject):
    """
    `Entity <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iottwinmaker-entity.html>`__
    """

    resource_type = "AWS::IoTTwinMaker::Entity"

    props: PropsDictType = {
        "Components": (dict, False),
        "CompositeComponents": (dict, False),
        "Description": (str, False),
        "EntityId": (str, False),
        "EntityName": (str, True),
        "ParentEntityId": (str, False),
        "Tags": (dict, False),
        "WorkspaceId": (str, True),
    }


class Scene(AWSObject):
    """
    `Scene <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iottwinmaker-scene.html>`__
    """

    resource_type = "AWS::IoTTwinMaker::Scene"

    props: PropsDictType = {
        "Capabilities": ([str], False),
        "ContentLocation": (str, True),
        "Description": (str, False),
        "SceneId": (str, True),
        "SceneMetadata": (dict, False),
        "Tags": (dict, False),
        "WorkspaceId": (str, True),
    }


class SyncJob(AWSObject):
    """
    `SyncJob <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iottwinmaker-syncjob.html>`__
    """

    resource_type = "AWS::IoTTwinMaker::SyncJob"

    props: PropsDictType = {
        "SyncRole": (str, True),
        "SyncSource": (str, True),
        "Tags": (dict, False),
        "WorkspaceId": (str, True),
    }


class Workspace(AWSObject):
    """
    `Workspace <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-iottwinmaker-workspace.html>`__
    """

    resource_type = "AWS::IoTTwinMaker::Workspace"

    props: PropsDictType = {
        "Description": (str, False),
        "Role": (str, True),
        "S3Location": (str, True),
        "Tags": (dict, False),
        "WorkspaceId": (str, True),
    }



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType
from .validators import integer


class InputParallelism(AWSProperty):
    """
    `InputParallelism <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-application-inputparallelism.html>`__
    """

    props: PropsDictType = {
        "Count": (integer, False),
    }


class InputLambdaProcessor(AWSProperty):
    """
    `InputLambdaProcessor <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-application-inputlambdaprocessor.html>`__
    """

    props: PropsDictType = {
        "ResourceARN": (str, True),
        "RoleARN": (str, True),
    }


class InputProcessingConfiguration(AWSProperty):
    """
    `InputProcessingConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-application-inputprocessingconfiguration.html>`__
    """

    props: PropsDictType = {
        "InputLambdaProcessor": (InputLambdaProcessor, False),
    }


class RecordColumn(AWSProperty):
    """
    `RecordColumn <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationreferencedatasource-recordcolumn.html>`__
    """

    props: PropsDictType = {
        "Mapping": (str, False),
        "Name": (str, True),
        "SqlType": (str, True),
    }


class CSVMappingParameters(AWSProperty):
    """
    `CSVMappingParameters <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationreferencedatasource-csvmappingparameters.html>`__
    """

    props: PropsDictType = {
        "RecordColumnDelimiter": (str, True),
        "RecordRowDelimiter": (str, True),
    }


class JSONMappingParameters(AWSProperty):
    """
    `JSONMappingParameters <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationreferencedatasource-jsonmappingparameters.html>`__
    """

    props: PropsDictType = {
        "RecordRowPath": (str, True),
    }


class MappingParameters(AWSProperty):
    """
    `MappingParameters <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationreferencedatasource-mappingparameters.html>`__
    """

    props: PropsDictType = {
        "CSVMappingParameters": (CSVMappingParameters, False),
        "JSONMappingParameters": (JSONMappingParameters, False),
    }


class RecordFormat(AWSProperty):
    """
    `RecordFormat <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationreferencedatasource-recordformat.html>`__
    """

    props: PropsDictType = {
        "MappingParameters": (MappingParameters, False),
        "RecordFormatType": (str, True),
    }


class InputSchema(AWSProperty):
    """
    `InputSchema <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-application-inputschema.html>`__
    """

    props: PropsDictType = {
        "RecordColumns": ([RecordColumn], True),
        "RecordEncoding": (str, False),
        "RecordFormat": (RecordFormat, True),
    }


class KinesisFirehoseInput(AWSProperty):
    """
    `KinesisFirehoseInput <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-application-kinesisfirehoseinput.html>`__
    """

    props: PropsDictType = {
        "ResourceARN": (str, True),
        "RoleARN": (str, True),
    }


class KinesisStreamsInput(AWSProperty):
    """
    `KinesisStreamsInput <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-application-kinesisstreamsinput.html>`__
    """

    props: PropsDictType = {
        "ResourceARN": (str, True),
        "RoleARN": (str, True),
    }


class Input(AWSProperty):
    """
    `Input <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-application-input.html>`__
    """

    props: PropsDictType = {
        "InputParallelism": (InputParallelism, False),
        "InputProcessingConfiguration": (InputProcessingConfiguration, False),
        "InputSchema": (InputSchema, True),
        "KinesisFirehoseInput": (KinesisFirehoseInput, False),
        "KinesisStreamsInput": (KinesisStreamsInput, False),
        "NamePrefix": (str, True),
    }


class Application(AWSObject):
    """
    `Application <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kinesisanalytics-application.html>`__
    """

    resource_type = "AWS::KinesisAnalytics::Application"

    props: PropsDictType = {
        "ApplicationCode": (str, False),
        "ApplicationDescription": (str, False),
        "ApplicationName": (str, False),
        "Inputs": ([Input], True),
    }


class DestinationSchema(AWSProperty):
    """
    `DestinationSchema <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationoutput-destinationschema.html>`__
    """

    props: PropsDictType = {
        "RecordFormatType": (str, False),
    }


class KinesisFirehoseOutput(AWSProperty):
    """
    `KinesisFirehoseOutput <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationoutput-kinesisfirehoseoutput.html>`__
    """

    props: PropsDictType = {
        "ResourceARN": (str, True),
        "RoleARN": (str, True),
    }


class KinesisStreamsOutput(AWSProperty):
    """
    `KinesisStreamsOutput <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationoutput-kinesisstreamsoutput.html>`__
    """

    props: PropsDictType = {
        "ResourceARN": (str, True),
        "RoleARN": (str, True),
    }


class LambdaOutput(AWSProperty):
    """
    `LambdaOutput <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationoutput-lambdaoutput.html>`__
    """

    props: PropsDictType = {
        "ResourceARN": (str, True),
        "RoleARN": (str, True),
    }


class Output(AWSProperty):
    """
    `Output <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationoutput-output.html>`__
    """

    props: PropsDictType = {
        "DestinationSchema": (DestinationSchema, True),
        "KinesisFirehoseOutput": (KinesisFirehoseOutput, False),
        "KinesisStreamsOutput": (KinesisStreamsOutput, False),
        "LambdaOutput": (LambdaOutput, False),
        "Name": (str, False),
    }


class ApplicationOutput(AWSObject):
    """
    `ApplicationOutput <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kinesisanalytics-applicationoutput.html>`__
    """

    resource_type = "AWS::KinesisAnalytics::ApplicationOutput"

    props: PropsDictType = {
        "ApplicationName": (str, True),
        "Output": (Output, True),
    }


class ReferenceSchema(AWSProperty):
    """
    `ReferenceSchema <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationreferencedatasource-referenceschema.html>`__
    """

    props: PropsDictType = {
        "RecordColumns": ([RecordColumn], True),
        "RecordEncoding": (str, False),
        "RecordFormat": (RecordFormat, True),
    }


class S3ReferenceDataSource(AWSProperty):
    """
    `S3ReferenceDataSource <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationreferencedatasource-s3referencedatasource.html>`__
    """

    props: PropsDictType = {
        "BucketARN": (str, True),
        "FileKey": (str, True),
        "ReferenceRoleARN": (str, True),
    }


class ReferenceDataSource(AWSProperty):
    """
    `ReferenceDataSource <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-kinesisanalytics-applicationreferencedatasource-referencedatasource.html>`__
    """

    props: PropsDictType = {
        "ReferenceSchema": (ReferenceSchema, True),
        "S3ReferenceDataSource": (S3ReferenceDataSource, False),
        "TableName": (str, False),
    }


class ApplicationReferenceDataSource(AWSObject):
    """
    `ApplicationReferenceDataSource <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-kinesisanalytics-applicationreferencedatasource.html>`__
    """

    resource_type = "AWS::KinesisAnalytics::ApplicationReferenceDataSource"

    props: PropsDictType = {
        "ApplicationName": (str, True),
        "ReferenceDataSource": (ReferenceDataSource, True),
    }



from awacs.aws import Allow, PolicyDocument, Principal, Statement
from awacs.sts import AssumeRole

from troposphere import Ref, Template
from troposphere.iam import InstanceProfile, Role

t = Template()

t.set_description(
    "AWS CloudFormation Sample Template: This template "
    "demonstrates the creation of IAM Roles and "
    "InstanceProfiles."
)

cfnrole = t.add_resource(
    Role(
        "CFNRole",
        AssumeRolePolicyDocument=PolicyDocument(
            Statement=[
                Statement(
                    Effect=Allow,
                    Action=[AssumeRole],
                    Principal=Principal("Service", ["ec2.amazonaws.com"]),
                )
            ]
        ),
    )
)

cfninstanceprofile = t.add_resource(
    InstanceProfile("CFNInstanceProfile", Roles=[Ref(cfnrole)])
)

print(t.to_json())



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType, Tags
from .validators import integer


class Accessor(AWSObject):
    """
    `Accessor <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-managedblockchain-accessor.html>`__
    """

    resource_type = "AWS::ManagedBlockchain::Accessor"

    props: PropsDictType = {
        "AccessorType": (str, True),
        "NetworkType": (str, False),
        "Tags": (Tags, False),
    }


class MemberFabricConfiguration(AWSProperty):
    """
    `MemberFabricConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-managedblockchain-member-memberfabricconfiguration.html>`__
    """

    props: PropsDictType = {
        "AdminPassword": (str, True),
        "AdminUsername": (str, True),
    }


class MemberFrameworkConfiguration(AWSProperty):
    """
    `MemberFrameworkConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-managedblockchain-member-memberframeworkconfiguration.html>`__
    """

    props: PropsDictType = {
        "MemberFabricConfiguration": (MemberFabricConfiguration, False),
    }


class MemberConfiguration(AWSProperty):
    """
    `MemberConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-managedblockchain-member-memberconfiguration.html>`__
    """

    props: PropsDictType = {
        "Description": (str, False),
        "MemberFrameworkConfiguration": (MemberFrameworkConfiguration, False),
        "Name": (str, True),
    }


class NetworkFabricConfiguration(AWSProperty):
    """
    `NetworkFabricConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-managedblockchain-member-networkfabricconfiguration.html>`__
    """

    props: PropsDictType = {
        "Edition": (str, True),
    }


class NetworkFrameworkConfiguration(AWSProperty):
    """
    `NetworkFrameworkConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-managedblockchain-member-networkframeworkconfiguration.html>`__
    """

    props: PropsDictType = {
        "NetworkFabricConfiguration": (NetworkFabricConfiguration, False),
    }


class ApprovalThresholdPolicy(AWSProperty):
    """
    `ApprovalThresholdPolicy <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-managedblockchain-member-approvalthresholdpolicy.html>`__
    """

    props: PropsDictType = {
        "ProposalDurationInHours": (integer, False),
        "ThresholdComparator": (str, False),
        "ThresholdPercentage": (integer, False),
    }


class VotingPolicy(AWSProperty):
    """
    `VotingPolicy <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-managedblockchain-member-votingpolicy.html>`__
    """

    props: PropsDictType = {
        "ApprovalThresholdPolicy": (ApprovalThresholdPolicy, False),
    }


class NetworkConfiguration(AWSProperty):
    """
    `NetworkConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-managedblockchain-member-networkconfiguration.html>`__
    """

    props: PropsDictType = {
        "Description": (str, False),
        "Framework": (str, True),
        "FrameworkVersion": (str, True),
        "Name": (str, True),
        "NetworkFrameworkConfiguration": (NetworkFrameworkConfiguration, False),
        "VotingPolicy": (VotingPolicy, True),
    }


class Member(AWSObject):
    """
    `Member <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-managedblockchain-member.html>`__
    """

    resource_type = "AWS::ManagedBlockchain::Member"

    props: PropsDictType = {
        "InvitationId": (str, False),
        "MemberConfiguration": (MemberConfiguration, True),
        "NetworkConfiguration": (NetworkConfiguration, False),
        "NetworkId": (str, False),
    }


class NodeConfiguration(AWSProperty):
    """
    `NodeConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-managedblockchain-node-nodeconfiguration.html>`__
    """

    props: PropsDictType = {
        "AvailabilityZone": (str, True),
        "InstanceType": (str, True),
    }


class Node(AWSObject):
    """
    `Node <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-managedblockchain-node.html>`__
    """

    resource_type = "AWS::ManagedBlockchain::Node"

    props: PropsDictType = {
        "MemberId": (str, False),
        "NetworkId": (str, True),
        "NodeConfiguration": (NodeConfiguration, True),
    }



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType, Tags
from .validators import boolean, integer


class CFNKubernetesAuditLogsConfiguration(AWSProperty):
    """
    `CFNKubernetesAuditLogsConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-guardduty-detector-cfnkubernetesauditlogsconfiguration.html>`__
    """

    props: PropsDictType = {
        "Enable": (boolean, True),
    }


class CFNKubernetesConfiguration(AWSProperty):
    """
    `CFNKubernetesConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-guardduty-detector-cfnkubernetesconfiguration.html>`__
    """

    props: PropsDictType = {
        "AuditLogs": (CFNKubernetesAuditLogsConfiguration, True),
    }


class CFNScanEc2InstanceWithFindingsConfiguration(AWSProperty):
    """
    `CFNScanEc2InstanceWithFindingsConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-guardduty-detector-cfnscanec2instancewithfindingsconfiguration.html>`__
    """

    props: PropsDictType = {
        "EbsVolumes": (boolean, False),
    }


class CFNMalwareProtectionConfiguration(AWSProperty):
    """
    `CFNMalwareProtectionConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-guardduty-detector-cfnmalwareprotectionconfiguration.html>`__
    """

    props: PropsDictType = {
        "ScanEc2InstanceWithFindings": (
            CFNScanEc2InstanceWithFindingsConfiguration,
            False,
        ),
    }


class CFNS3LogsConfiguration(AWSProperty):
    """
    `CFNS3LogsConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-guardduty-detector-cfns3logsconfiguration.html>`__
    """

    props: PropsDictType = {
        "Enable": (boolean, True),
    }


class CFNDataSourceConfigurations(AWSProperty):
    """
    `CFNDataSourceConfigurations <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-guardduty-detector-cfndatasourceconfigurations.html>`__
    """

    props: PropsDictType = {
        "Kubernetes": (CFNKubernetesConfiguration, False),
        "MalwareProtection": (CFNMalwareProtectionConfiguration, False),
        "S3Logs": (CFNS3LogsConfiguration, False),
    }


class CFNFeatureAdditionalConfiguration(AWSProperty):
    """
    `CFNFeatureAdditionalConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-guardduty-detector-cfnfeatureadditionalconfiguration.html>`__
    """

    props: PropsDictType = {
        "Name": (str, False),
        "Status": (str, False),
    }


class CFNFeatureConfiguration(AWSProperty):
    """
    `CFNFeatureConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-guardduty-detector-cfnfeatureconfiguration.html>`__
    """

    props: PropsDictType = {
        "AdditionalConfiguration": ([CFNFeatureAdditionalConfiguration], False),
        "Name": (str, True),
        "Status": (str, True),
    }


class TagItem(AWSProperty):
    """
    `TagItem <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-guardduty-threatintelset-tagitem.html>`__
    """

    props: PropsDictType = {
        "Key": (str, True),
        "Value": (str, True),
    }


class Detector(AWSObject):
    """
    `Detector <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-guardduty-detector.html>`__
    """

    resource_type = "AWS::GuardDuty::Detector"

    props: PropsDictType = {
        "DataSources": (CFNDataSourceConfigurations, False),
        "Enable": (boolean, True),
        "Features": ([CFNFeatureConfiguration], False),
        "FindingPublishingFrequency": (str, False),
        "Tags": ([TagItem], False),
    }


class Condition(AWSProperty):
    """
    `Condition <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-guardduty-filter-condition.html>`__
    """

    props: PropsDictType = {
        "Eq": ([str], False),
        "Equals": ([str], False),
        "GreaterThan": (integer, False),
        "GreaterThanOrEqual": (integer, False),
        "Gt": (integer, False),
        "Gte": (integer, False),
        "LessThan": (integer, False),
        "LessThanOrEqual": (integer, False),
        "Lt": (integer, False),
        "Lte": (integer, False),
        "Neq": ([str], False),
        "NotEquals": ([str], False),
    }


class FindingCriteria(AWSProperty):
    """
    `FindingCriteria <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-guardduty-filter-findingcriteria.html>`__
    """

    props: PropsDictType = {
        "Criterion": (dict, False),
    }


class Filter(AWSObject):
    """
    `Filter <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-guardduty-filter.html>`__
    """

    resource_type = "AWS::GuardDuty::Filter"

    props: PropsDictType = {
        "Action": (str, False),
        "Description": (str, False),
        "DetectorId": (str, False),
        "FindingCriteria": (FindingCriteria, True),
        "Name": (str, False),
        "Rank": (integer, False),
        "Tags": (Tags, False),
    }


class IPSet(AWSObject):
    """
    `IPSet <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-guardduty-ipset.html>`__
    """

    resource_type = "AWS::GuardDuty::IPSet"

    props: PropsDictType = {
        "Activate": (boolean, False),
        "DetectorId": (str, False),
        "Format": (str, True),
        "Location": (str, True),
        "Name": (str, False),
        "Tags": ([TagItem], False),
    }


class Master(AWSObject):
    """
    `Master <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-guardduty-master.html>`__
    """

    resource_type = "AWS::GuardDuty::Master"

    props: PropsDictType = {
        "DetectorId": (str, True),
        "InvitationId": (str, False),
        "MasterId": (str, True),
    }


class Member(AWSObject):
    """
    `Member <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-guardduty-member.html>`__
    """

    resource_type = "AWS::GuardDuty::Member"

    props: PropsDictType = {
        "DetectorId": (str, True),
        "DisableEmailNotification": (boolean, False),
        "Email": (str, True),
        "MemberId": (str, True),
        "Message": (str, False),
        "Status": (str, False),
    }


class ThreatIntelSet(AWSObject):
    """
    `ThreatIntelSet <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-guardduty-threatintelset.html>`__
    """

    resource_type = "AWS::GuardDuty::ThreatIntelSet"

    props: PropsDictType = {
        "Activate": (boolean, False),
        "DetectorId": (str, False),
        "Format": (str, True),
        "Location": (str, True),
        "Name": (str, False),
        "Tags": ([TagItem], False),
    }



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.


def component_platforms(component_platform):
    """
    Property: Component.Platform
    """
    valid_component_platforms = ["Linux", "Windows"]
    if component_platform not in valid_component_platforms:
        raise ValueError(
            'Platform must be one of: "%s"' % (", ".join(valid_component_platforms))
        )
    return component_platform


def ebsinstanceblockdevicespecification_volume_type(type):
    """
    Property: EbsInstanceBlockDeviceSpecification.VolumeType
    """
    valid_types = ["gp2", "gp3", "io1", "io2", "sc1", "st1", "standard"]
    if type not in valid_types:
        raise ValueError('VolumeType must be one of: "%s"' % (", ".join(valid_types)))
    return type


def imagepipeline_status(status):
    """
    Property: ImagePipeline.Status
    """
    valid_status = ["DISABLED", "ENABLED"]
    if status not in valid_status:
        raise ValueError('Status must be one of: "%s"' % (", ".join(valid_status)))
    return status


def schedule_pipelineexecutionstartcondition(startcondition):
    """
    Property: Schedule.PipelineExecutionStartCondition
    """
    valid_startcondition = [
        "EXPRESSION_MATCH_AND_DEPENDENCY_UPDATES_AVAILABLE",
        "EXPRESSION_MATCH_ONLY",
    ]
    if startcondition not in valid_startcondition:
        raise ValueError(
            'PipelineExecutionStartCondition must be one of: "%s"'
            % (", ".join(valid_startcondition))
        )
    return startcondition



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType


class ControlCondition(AWSProperty):
    """
    `ControlCondition <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-arczonalshift-zonalautoshiftconfiguration-controlcondition.html>`__
    """

    props: PropsDictType = {
        "AlarmIdentifier": (str, True),
        "Type": (str, True),
    }


class PracticeRunConfiguration(AWSProperty):
    """
    `PracticeRunConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-arczonalshift-zonalautoshiftconfiguration-practicerunconfiguration.html>`__
    """

    props: PropsDictType = {
        "BlockedDates": ([str], False),
        "BlockedWindows": ([str], False),
        "BlockingAlarms": ([ControlCondition], False),
        "OutcomeAlarms": ([ControlCondition], True),
    }


class ZonalAutoshiftConfiguration(AWSObject):
    """
    `ZonalAutoshiftConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-arczonalshift-zonalautoshiftconfiguration.html>`__
    """

    resource_type = "AWS::ARCZonalShift::ZonalAutoshiftConfiguration"

    props: PropsDictType = {
        "PracticeRunConfiguration": (PracticeRunConfiguration, False),
        "ResourceIdentifier": (str, False),
        "ZonalAutoshiftStatus": (str, False),
    }



import unittest

import troposphere.cloudfront as cloudfront
from troposphere import If


class TestCorsConfig(unittest.TestCase):
    def test_allowmethods(self):
        cloudfront.AccessControlAllowMethods(Items=[])
        cloudfront.AccessControlAllowMethods(Items=["GET", "POST"])
        cloudfront.AccessControlAllowMethods(
            Items=If("SomeCondition", ["GET"], ["POST"])
        )
        with self.assertRaises(ValueError):
            cloudfront.AccessControlAllowMethods(Items=["get"])
        with self.assertRaises(ValueError):
            cloudfront.AccessControlAllowMethods(Items=["GET", "get", "PUT"])
        with self.assertRaises(TypeError):
            cloudfront.AccessControlAllowMethods(Items=10)
        with self.assertRaises(TypeError):
            cloudfront.AccessControlAllowMethods(Items="GET")


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType, Tags
from .validators import integer


class Agent(AWSObject):
    """
    `Agent <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-datasync-agent.html>`__
    """

    resource_type = "AWS::DataSync::Agent"

    props: PropsDictType = {
        "ActivationKey": (str, False),
        "AgentName": (str, False),
        "SecurityGroupArns": ([str], False),
        "SubnetArns": ([str], False),
        "Tags": (Tags, False),
        "VpcEndpointId": (str, False),
    }


class AzureBlobSasConfiguration(AWSProperty):
    """
    `AzureBlobSasConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-datasync-locationazureblob-azureblobsasconfiguration.html>`__
    """

    props: PropsDictType = {
        "AzureBlobSasToken": (str, True),
    }


class LocationAzureBlob(AWSObject):
    """
    `LocationAzureBlob <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-datasync-locationazureblob.html>`__
    """

    resource_type = "AWS::DataSync::LocationAzureBlob"

    props: PropsDictType = {
        "AgentArns": ([str], True),
        "AzureAccessTier": (str, False),
        "AzureBlobAuthenticationType": (str, True),
        "AzureBlobContainerUrl": (str, False),
        "AzureBlobSasConfiguration": (AzureBlobSasConfiguration, False),
        "AzureBlobType": (str, False),
        "Subdirectory": (str, False),
        "Tags": (Tags, False),
    }


class Ec2Config(AWSProperty):
    """
    `Ec2Config <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-datasync-locationefs-ec2config.html>`__
    """

    props: PropsDictType = {
        "SecurityGroupArns": ([str], True),
        "SubnetArn": (str, True),
    }


class LocationEFS(AWSObject):
    """
    `LocationEFS <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-datasync-locationefs.html>`__
    """

    resource_type = "AWS::DataSync::LocationEFS"

    props: PropsDictType = {
        "AccessPointArn": (str, False),
        "Ec2Config": (Ec2Config, True),
        "EfsFilesystemArn": (str, False),
        "FileSystemAccessRoleArn": (str, False),
        "InTransitEncryption": (str, False),
        "Subdirectory": (str, False),
        "Tags": (Tags, False),
    }


class LocationFSxLustre(AWSObject):
    """
    `LocationFSxLustre <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-datasync-locationfsxlustre.html>`__
    """

    resource_type = "AWS::DataSync::LocationFSxLustre"

    props: PropsDictType = {
        "FsxFilesystemArn": (str, False),
        "SecurityGroupArns": ([str], True),
        "Subdirectory": (str, False),
        "Tags": (Tags, False),
    }


class MountOptions(AWSProperty):
    """
    `MountOptions <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-datasync-locationsmb-mountoptions.html>`__
    """

    props: PropsDictType = {
        "Version": (str, False),
    }


class NFS(AWSProperty):
    """
    `NFS <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-datasync-locationfsxopenzfs-nfs.html>`__
    """

    props: PropsDictType = {
        "MountOptions": (MountOptions, True),
    }


class SmbMountOptions(AWSProperty):
    """
    `SmbMountOptions <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-datasync-locationfsxontap-smbmountoptions.html>`__
    """

    props: PropsDictType = {
        "Version": (str, False),
    }


class SMB(AWSProperty):
    """
    `SMB <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-datasync-locationfsxontap-smb.html>`__
    """

    props: PropsDictType = {
        "Domain": (str, False),
        "MountOptions": (SmbMountOptions, True),
        "Password": (str, True),
        "User": (str, True),
    }


class ONTAPProtocol(AWSProperty):
    """
    `ONTAPProtocol <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-datasync-locationfsxontap-protocol.html>`__
    """

    props: PropsDictType = {
        "NFS": (NFS, False),
        "SMB": (SMB, False),
    }


class LocationFSxONTAP(AWSObject):
    """
    `LocationFSxONTAP <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-datasync-locationfsxontap.html>`__
    """

    resource_type = "AWS::DataSync::LocationFSxONTAP"

    props: PropsDictType = {
        "Protocol": (ONTAPProtocol, False),
        "SecurityGroupArns": ([str], True),
        "StorageVirtualMachineArn": (str, True),
        "Subdirectory": (str, False),
        "Tags": (Tags, False),
    }


class Protocol(AWSProperty):
    """
    `Protocol <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-datasync-locationfsxopenzfs-protocol.html>`__
    """

    props: PropsDictType = {
        "NFS": (NFS, False),
    }


class LocationFSxOpenZFS(AWSObject):
    """
    `LocationFSxOpenZFS <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-datasync-locationfsxopenzfs.html>`__
    """

    resource_type = "AWS::DataSync::LocationFSxOpenZFS"

    props: PropsDictType = {
        "FsxFilesystemArn": (str, False),
        "Protocol": (Protocol, True),
        "SecurityGroupArns": ([str], True),
        "Subdirectory": (str, False),
        "Tags": (Tags, False),
    }


class LocationFSxWindows(AWSObject):
    """
    `LocationFSxWindows <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-datasync-locationfsxwindows.html>`__
    """

    resource_type = "AWS::DataSync::LocationFSxWindows"

    props: PropsDictType = {
        "Domain": (str, False),
        "FsxFilesystemArn": (str, False),
        "Password": (str, False),
        "SecurityGroupArns": ([str], True),
        "Subdirectory": (str, False),
        "Tags": (Tags, False),
        "User": (str, True),
    }


class NameNode(AWSProperty):
    """
    `NameNode <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-datasync-locationhdfs-namenode.html>`__
    """

    props: PropsDictType = {
        "Hostname": (str, True),
        "Port": (integer, True),
    }


class QopConfiguration(AWSProperty):
    """
    `QopConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-datasync-locationhdfs-qopconfiguration.html>`__
    """

    props: PropsDictType = {
        "DataTransferProtection": (str, False),
        "RpcProtection": (str, False),
    }


class LocationHDFS(AWSObject):
    """
    `LocationHDFS <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-datasync-locationhdfs.html>`__
    """

    resource_type = "AWS::DataSync::LocationHDFS"

    props: PropsDictType = {
        "AgentArns": ([str], True),
        "AuthenticationType": (str, True),
        "BlockSize": (integer, False),
        "KerberosKeytab": (str, False),
        "KerberosKrb5Conf": (str, False),
        "KerberosPrincipal": (str, False),
        "KmsKeyProviderUri": (str, False),
        "NameNodes": ([NameNode], True),
        "QopConfiguration": (QopConfiguration, False),
        "ReplicationFactor": (integer, False),
        "SimpleUser": (str, False),
        "Subdirectory": (str, False),
        "Tags": (Tags, False),
    }


class OnPremConfig(AWSProperty):
    """
    `OnPremConfig <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-datasync-locationnfs-onpremconfig.html>`__
    """

    props: PropsDictType = {
        "AgentArns": ([str], True),
    }


class LocationNFS(AWSObject):
    """
    `LocationNFS <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-datasync-locationnfs.html>`__
    """

    resource_type = "AWS::DataSync::LocationNFS"

    props: PropsDictType = {
        "MountOptions": (MountOptions, False),
        "OnPremConfig": (OnPremConfig, True),
        "ServerHostname": (str, False),
        "Subdirectory": (str, False),
        "Tags": (Tags, False),
    }


class LocationObjectStorage(AWSObject):
    """
    `LocationObjectStorage <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-datasync-locationobjectstorage.html>`__
    """

    resource_type = "AWS::DataSync::LocationObjectStorage"

    props: PropsDictType = {
        "AccessKey": (str, False),
        "AgentArns": ([str], True),
        "BucketName": (str, False),
        "SecretKey": (str, False),
        "ServerCertificate": (str, False),
        "ServerHostname": (str, False),
        "ServerPort": (integer, False),
        "ServerProtocol": (str, False),
        "Subdirectory": (str, False),
        "Tags": (Tags, False),
    }


class S3Config(AWSProperty):
    """
    `S3Config <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-datasync-locations3-s3config.html>`__
    """

    props: PropsDictType = {
        "BucketAccessRoleArn": (str, True),
    }


class LocationS3(AWSObject):
    """
    `LocationS3 <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-datasync-locations3.html>`__
    """

    resource_type = "AWS::DataSync::LocationS3"

    props: PropsDictType = {
        "S3BucketArn": (str, False),
        "S3Config": (S3Config, True),
        "S3StorageClass": (str, False),
        "Subdirectory": (str, False),
        "Tags": (Tags, False),
    }


class LocationSMB(AWSObject):
    """
    `LocationSMB <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-datasync-locationsmb.html>`__
    """

    resource_type = "AWS::DataSync::LocationSMB"

    props: PropsDictType = {
        "AgentArns": ([str], True),
        "Domain": (str, False),
        "MountOptions": (MountOptions, False),
        "Password": (str, False),
        "ServerHostname": (str, False),
        "Subdirectory": (str, False),
        "Tags": (Tags, False),
        "User": (str, True),
    }


class ServerConfiguration(AWSProperty):
    """
    `ServerConfiguration <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-datasync-storagesystem-serverconfiguration.html>`__
    """

    props: PropsDictType = {
        "ServerHostname": (str, True),
        "ServerPort": (integer, False),
    }


class ServerCredentials(AWSProperty):
    """
    `ServerCredentials <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-datasync-storagesystem-servercredentials.html>`__
    """

    props: PropsDictType = {
        "Password": (str, True),
        "Username": (str, True),
    }


class StorageSystem(AWSObject):
    """
    `StorageSystem <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-datasync-storagesystem.html>`__
    """

    resource_type = "AWS::DataSync::StorageSystem"

    props: PropsDictType = {
        "AgentArns": ([str], True),
        "CloudWatchLogGroupArn": (str, False),
        "Name": (str, False),
        "ServerConfiguration": (ServerConfiguration, True),
        "ServerCredentials": (ServerCredentials, False),
        "SystemType": (str, True),
        "Tags": (Tags, False),
    }


class FilterRule(AWSProperty):
    """
    `FilterRule <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-datasync-task-filterrule.html>`__
    """

    props: PropsDictType = {
        "FilterType": (str, False),
        "Value": (str, False),
    }


class Options(AWSProperty):
    """
    `Options <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-datasync-task-options.html>`__
    """

    props: PropsDictType = {
        "Atime": (str, False),
        "BytesPerSecond": (integer, False),
        "Gid": (str, False),
        "LogLevel": (str, False),
        "Mtime": (str, False),
        "ObjectTags": (str, False),
        "OverwriteMode": (str, False),
        "PosixPermissions": (str, False),
        "PreserveDeletedFiles": (str, False),
        "PreserveDevices": (str, False),
        "SecurityDescriptorCopyFlags": (str, False),
        "TaskQueueing": (str, False),
        "TransferMode": (str, False),
        "Uid": (str, False),
        "VerifyMode": (str, False),
    }


class S3(AWSProperty):
    """
    `S3 <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-datasync-task-s3.html>`__
    """

    props: PropsDictType = {
        "BucketAccessRoleArn": (str, False),
        "S3BucketArn": (str, False),
        "Subdirectory": (str, False),
    }


class Destination(AWSProperty):
    """
    `Destination <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-datasync-task-destination.html>`__
    """

    props: PropsDictType = {
        "S3": (S3, False),
    }


class Deleted(AWSProperty):
    """
    `Deleted <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-datasync-task-deleted.html>`__
    """

    props: PropsDictType = {
        "ReportLevel": (str, False),
    }


class Skipped(AWSProperty):
    """
    `Skipped <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-datasync-task-skipped.html>`__
    """

    props: PropsDictType = {
        "ReportLevel": (str, False),
    }


class Transferred(AWSProperty):
    """
    `Transferred <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-datasync-task-transferred.html>`__
    """

    props: PropsDictType = {
        "ReportLevel": (str, False),
    }


class Verified(AWSProperty):
    """
    `Verified <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-datasync-task-verified.html>`__
    """

    props: PropsDictType = {
        "ReportLevel": (str, False),
    }


class Overrides(AWSProperty):
    """
    `Overrides <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-datasync-task-overrides.html>`__
    """

    props: PropsDictType = {
        "Deleted": (Deleted, False),
        "Skipped": (Skipped, False),
        "Transferred": (Transferred, False),
        "Verified": (Verified, False),
    }


class TaskReportConfig(AWSProperty):
    """
    `TaskReportConfig <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-datasync-task-taskreportconfig.html>`__
    """

    props: PropsDictType = {
        "Destination": (Destination, True),
        "ObjectVersionIds": (str, False),
        "OutputType": (str, True),
        "Overrides": (Overrides, False),
        "ReportLevel": (str, False),
    }


class TaskSchedule(AWSProperty):
    """
    `TaskSchedule <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-datasync-task-taskschedule.html>`__
    """

    props: PropsDictType = {
        "ScheduleExpression": (str, True),
    }


class Task(AWSObject):
    """
    `Task <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-datasync-task.html>`__
    """

    resource_type = "AWS::DataSync::Task"

    props: PropsDictType = {
        "CloudWatchLogGroupArn": (str, False),
        "DestinationLocationArn": (str, True),
        "Excludes": ([FilterRule], False),
        "Includes": ([FilterRule], False),
        "Name": (str, False),
        "Options": (Options, False),
        "Schedule": (TaskSchedule, False),
        "SourceLocationArn": (str, True),
        "Tags": (Tags, False),
        "TaskReportConfig": (TaskReportConfig, False),
    }


class NfsMountOptions(AWSProperty):
    """
    `NfsMountOptions <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-datasync-locationfsxontap-nfsmountoptions.html>`__
    """

    props: PropsDictType = {
        "Version": (str, False),
    }



# Copyright (c) 2012-2021, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.


def validate_rule_group_type(rule_group_type):
    """
    Validate Type for RuleGroup
    Property: RuleGroup.Type
    """

    VALID_RULE_GROUP_TYPES = ("STATEFUL", "STATELESS")
    if rule_group_type not in VALID_RULE_GROUP_TYPES:
        raise ValueError(
            "RuleGroup Type must be one of %s" % ", ".join(VALID_RULE_GROUP_TYPES)
        )
    return rule_group_type



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, PropsDictType, Tags
from .validators import boolean


class Permission(AWSObject):
    """
    `Permission <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ram-permission.html>`__
    """

    resource_type = "AWS::RAM::Permission"

    props: PropsDictType = {
        "Name": (str, True),
        "PolicyTemplate": (dict, True),
        "ResourceType": (str, True),
        "Tags": (Tags, False),
    }


class ResourceShare(AWSObject):
    """
    `ResourceShare <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-ram-resourceshare.html>`__
    """

    resource_type = "AWS::RAM::ResourceShare"

    props: PropsDictType = {
        "AllowExternalPrincipals": (boolean, False),
        "Name": (str, True),
        "PermissionArns": ([str], False),
        "Principals": ([str], False),
        "ResourceArns": ([str], False),
        "Sources": ([str], False),
        "Tags": (Tags, False),
    }



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType
from .validators import boolean, double, integer


class DataSource(AWSProperty):
    """
    `DataSource <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-personalize-dataset-datasource.html>`__
    """

    props: PropsDictType = {
        "DataLocation": (str, False),
    }


class DatasetImportJob(AWSProperty):
    """
    `DatasetImportJob <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-personalize-dataset-datasetimportjob.html>`__
    """

    props: PropsDictType = {
        "DataSource": (DataSource, False),
        "DatasetArn": (str, False),
        "DatasetImportJobArn": (str, False),
        "JobName": (str, False),
        "RoleArn": (str, False),
    }


class Dataset(AWSObject):
    """
    `Dataset <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-personalize-dataset.html>`__
    """

    resource_type = "AWS::Personalize::Dataset"

    props: PropsDictType = {
        "DatasetGroupArn": (str, True),
        "DatasetImportJob": (DatasetImportJob, False),
        "DatasetType": (str, True),
        "Name": (str, True),
        "SchemaArn": (str, True),
    }


class DatasetGroup(AWSObject):
    """
    `DatasetGroup <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-personalize-datasetgroup.html>`__
    """

    resource_type = "AWS::Personalize::DatasetGroup"

    props: PropsDictType = {
        "Domain": (str, False),
        "KmsKeyArn": (str, False),
        "Name": (str, True),
        "RoleArn": (str, False),
    }


class Schema(AWSObject):
    """
    `Schema <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-personalize-schema.html>`__
    """

    resource_type = "AWS::Personalize::Schema"

    props: PropsDictType = {
        "Domain": (str, False),
        "Name": (str, True),
        "Schema": (str, True),
    }


class AutoMLConfig(AWSProperty):
    """
    `AutoMLConfig <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-personalize-solution-automlconfig.html>`__
    """

    props: PropsDictType = {
        "MetricName": (str, False),
        "RecipeList": ([str], False),
    }


class CategoricalHyperParameterRange(AWSProperty):
    """
    `CategoricalHyperParameterRange <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-personalize-solution-categoricalhyperparameterrange.html>`__
    """

    props: PropsDictType = {
        "Name": (str, False),
        "Values": ([str], False),
    }


class ContinuousHyperParameterRange(AWSProperty):
    """
    `ContinuousHyperParameterRange <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-personalize-solution-continuoushyperparameterrange.html>`__
    """

    props: PropsDictType = {
        "MaxValue": (double, False),
        "MinValue": (double, False),
        "Name": (str, False),
    }


class IntegerHyperParameterRange(AWSProperty):
    """
    `IntegerHyperParameterRange <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-personalize-solution-integerhyperparameterrange.html>`__
    """

    props: PropsDictType = {
        "MaxValue": (integer, False),
        "MinValue": (integer, False),
        "Name": (str, False),
    }


class AlgorithmHyperParameterRanges(AWSProperty):
    """
    `AlgorithmHyperParameterRanges <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-personalize-solution-algorithmhyperparameterranges.html>`__
    """

    props: PropsDictType = {
        "CategoricalHyperParameterRanges": ([CategoricalHyperParameterRange], False),
        "ContinuousHyperParameterRanges": ([ContinuousHyperParameterRange], False),
        "IntegerHyperParameterRanges": ([IntegerHyperParameterRange], False),
    }


class HpoObjective(AWSProperty):
    """
    `HpoObjective <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-personalize-solution-hpoobjective.html>`__
    """

    props: PropsDictType = {
        "MetricName": (str, False),
        "MetricRegex": (str, False),
        "Type": (str, False),
    }


class HpoResourceConfig(AWSProperty):
    """
    `HpoResourceConfig <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-personalize-solution-hporesourceconfig.html>`__
    """

    props: PropsDictType = {
        "MaxNumberOfTrainingJobs": (str, False),
        "MaxParallelTrainingJobs": (str, False),
    }


class HpoConfig(AWSProperty):
    """
    `HpoConfig <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-personalize-solution-hpoconfig.html>`__
    """

    props: PropsDictType = {
        "AlgorithmHyperParameterRanges": (AlgorithmHyperParameterRanges, False),
        "HpoObjective": (HpoObjective, False),
        "HpoResourceConfig": (HpoResourceConfig, False),
    }


class SolutionConfig(AWSProperty):
    """
    `SolutionConfig <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-personalize-solution-solutionconfig.html>`__
    """

    props: PropsDictType = {
        "AlgorithmHyperParameters": (dict, False),
        "AutoMLConfig": (AutoMLConfig, False),
        "EventValueThreshold": (str, False),
        "FeatureTransformationParameters": (dict, False),
        "HpoConfig": (HpoConfig, False),
    }


class Solution(AWSObject):
    """
    `Solution <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-personalize-solution.html>`__
    """

    resource_type = "AWS::Personalize::Solution"

    props: PropsDictType = {
        "DatasetGroupArn": (str, True),
        "EventType": (str, False),
        "Name": (str, True),
        "PerformAutoML": (boolean, False),
        "PerformHPO": (boolean, False),
        "RecipeArn": (str, False),
        "SolutionConfig": (SolutionConfig, False),
    }



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType, Tags
from .validators import boolean


class EntityTypeProperty(AWSProperty):
    """
    `EntityTypeProperty <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-frauddetector-eventtype-entitytype.html>`__
    """

    props: PropsDictType = {
        "Arn": (str, False),
        "CreatedTime": (str, False),
        "Description": (str, False),
        "Inline": (boolean, False),
        "LastUpdatedTime": (str, False),
        "Name": (str, False),
        "Tags": (Tags, False),
    }


class EventVariable(AWSProperty):
    """
    `EventVariable <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-frauddetector-eventtype-eventvariable.html>`__
    """

    props: PropsDictType = {
        "Arn": (str, False),
        "CreatedTime": (str, False),
        "DataSource": (str, False),
        "DataType": (str, False),
        "DefaultValue": (str, False),
        "Description": (str, False),
        "Inline": (boolean, False),
        "LastUpdatedTime": (str, False),
        "Name": (str, False),
        "Tags": (Tags, False),
        "VariableType": (str, False),
    }


class LabelProperty(AWSProperty):
    """
    `LabelProperty <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-frauddetector-eventtype-label.html>`__
    """

    props: PropsDictType = {
        "Arn": (str, False),
        "CreatedTime": (str, False),
        "Description": (str, False),
        "Inline": (boolean, False),
        "LastUpdatedTime": (str, False),
        "Name": (str, False),
        "Tags": (Tags, False),
    }


class EventTypeProperty(AWSProperty):
    """
    `EventTypeProperty <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-frauddetector-detector-eventtype.html>`__
    """

    props: PropsDictType = {
        "Arn": (str, False),
        "CreatedTime": (str, False),
        "Description": (str, False),
        "EntityTypes": ([EntityTypeProperty], False),
        "EventVariables": ([EventVariable], False),
        "Inline": (boolean, False),
        "Labels": ([LabelProperty], False),
        "LastUpdatedTime": (str, False),
        "Name": (str, False),
        "Tags": (Tags, False),
    }


class Model(AWSProperty):
    """
    `Model <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-frauddetector-detector-model.html>`__
    """

    props: PropsDictType = {
        "Arn": (str, False),
    }


class OutcomeProperty(AWSProperty):
    """
    `OutcomeProperty <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-frauddetector-detector-outcome.html>`__
    """

    props: PropsDictType = {
        "Arn": (str, False),
        "CreatedTime": (str, False),
        "Description": (str, False),
        "Inline": (boolean, False),
        "LastUpdatedTime": (str, False),
        "Name": (str, False),
        "Tags": (Tags, False),
    }


class Rule(AWSProperty):
    """
    `Rule <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-frauddetector-detector-rule.html>`__
    """

    props: PropsDictType = {
        "Arn": (str, False),
        "CreatedTime": (str, False),
        "Description": (str, False),
        "DetectorId": (str, False),
        "Expression": (str, False),
        "Language": (str, False),
        "LastUpdatedTime": (str, False),
        "Outcomes": ([OutcomeProperty], False),
        "RuleId": (str, False),
        "RuleVersion": (str, False),
        "Tags": (Tags, False),
    }


class Detector(AWSObject):
    """
    `Detector <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-frauddetector-detector.html>`__
    """

    resource_type = "AWS::FraudDetector::Detector"

    props: PropsDictType = {
        "AssociatedModels": ([Model], False),
        "Description": (str, False),
        "DetectorId": (str, True),
        "DetectorVersionStatus": (str, False),
        "EventType": (EventTypeProperty, True),
        "RuleExecutionMode": (str, False),
        "Rules": ([Rule], True),
        "Tags": (Tags, False),
    }


class EntityType(AWSObject):
    """
    `EntityType <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-frauddetector-entitytype.html>`__
    """

    resource_type = "AWS::FraudDetector::EntityType"

    props: PropsDictType = {
        "Description": (str, False),
        "Name": (str, True),
        "Tags": (Tags, False),
    }


class EventType(AWSObject):
    """
    `EventType <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-frauddetector-eventtype.html>`__
    """

    resource_type = "AWS::FraudDetector::EventType"

    props: PropsDictType = {
        "Description": (str, False),
        "EntityTypes": ([EntityTypeProperty], True),
        "EventVariables": ([EventVariable], True),
        "Labels": ([LabelProperty], True),
        "Name": (str, True),
        "Tags": (Tags, False),
    }


class Label(AWSObject):
    """
    `Label <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-frauddetector-label.html>`__
    """

    resource_type = "AWS::FraudDetector::Label"

    props: PropsDictType = {
        "Description": (str, False),
        "Name": (str, True),
        "Tags": (Tags, False),
    }


class List(AWSObject):
    """
    `List <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-frauddetector-list.html>`__
    """

    resource_type = "AWS::FraudDetector::List"

    props: PropsDictType = {
        "Description": (str, False),
        "Elements": ([str], False),
        "Name": (str, True),
        "Tags": (Tags, False),
        "VariableType": (str, False),
    }


class Outcome(AWSObject):
    """
    `Outcome <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-frauddetector-outcome.html>`__
    """

    resource_type = "AWS::FraudDetector::Outcome"

    props: PropsDictType = {
        "Description": (str, False),
        "Name": (str, True),
        "Tags": (Tags, False),
    }


class Variable(AWSObject):
    """
    `Variable <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-frauddetector-variable.html>`__
    """

    resource_type = "AWS::FraudDetector::Variable"

    props: PropsDictType = {
        "DataSource": (str, True),
        "DataType": (str, True),
        "DefaultValue": (str, True),
        "Description": (str, False),
        "Name": (str, True),
        "Tags": (Tags, False),
        "VariableType": (str, False),
    }



# Copyright (c) 2012-2021, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.


from . import tags_or_list


def kinesis_stream_mode(mode):
    """
    Property: StreamModeDetails.StreamMode
    """
    valid_modes = ["ON_DEMAND", "PROVISIONED"]
    if mode not in valid_modes:
        raise ValueError('ContentType must be one of: "%s"' % (", ".join(valid_modes)))
    return mode


def validate_tags_or_list(x):
    """
    Property: Stream.Tags
    """
    return tags_or_list(x)



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.


from ..compat import validate_policytype


def policytypes(policy):
    """
    Property: TopicPolicy.PolicyDocument
    """
    return validate_policytype(policy)



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType, Tags
from .validators import boolean, double, integer


class Alarm(AWSObject):
    """
    `Alarm <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lightsail-alarm.html>`__
    """

    resource_type = "AWS::Lightsail::Alarm"

    props: PropsDictType = {
        "AlarmName": (str, True),
        "ComparisonOperator": (str, True),
        "ContactProtocols": ([str], False),
        "DatapointsToAlarm": (integer, False),
        "EvaluationPeriods": (integer, True),
        "MetricName": (str, True),
        "MonitoredResourceName": (str, True),
        "NotificationEnabled": (boolean, False),
        "NotificationTriggers": ([str], False),
        "Threshold": (double, True),
        "TreatMissingData": (str, False),
    }


class AccessRules(AWSProperty):
    """
    `AccessRules <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lightsail-bucket-accessrules.html>`__
    """

    props: PropsDictType = {
        "AllowPublicOverrides": (boolean, False),
        "GetObject": (str, False),
    }


class Bucket(AWSObject):
    """
    `Bucket <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lightsail-bucket.html>`__
    """

    resource_type = "AWS::Lightsail::Bucket"

    props: PropsDictType = {
        "AccessRules": (AccessRules, False),
        "BucketName": (str, True),
        "BundleId": (str, True),
        "ObjectVersioning": (boolean, False),
        "ReadOnlyAccessAccounts": ([str], False),
        "ResourcesReceivingAccess": ([str], False),
        "Tags": (Tags, False),
    }


class Certificate(AWSObject):
    """
    `Certificate <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lightsail-certificate.html>`__
    """

    resource_type = "AWS::Lightsail::Certificate"

    props: PropsDictType = {
        "CertificateName": (str, True),
        "DomainName": (str, True),
        "SubjectAlternativeNames": ([str], False),
        "Tags": (Tags, False),
    }


class EnvironmentVariable(AWSProperty):
    """
    `EnvironmentVariable <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lightsail-container-environmentvariable.html>`__
    """

    props: PropsDictType = {
        "Value": (str, False),
        "Variable": (str, False),
    }


class PortInfo(AWSProperty):
    """
    `PortInfo <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lightsail-container-portinfo.html>`__
    """

    props: PropsDictType = {
        "Port": (str, False),
        "Protocol": (str, False),
    }


class ContainerProperty(AWSProperty):
    """
    `ContainerProperty <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lightsail-container-container.html>`__
    """

    props: PropsDictType = {
        "Command": ([str], False),
        "ContainerName": (str, False),
        "Environment": ([EnvironmentVariable], False),
        "Image": (str, False),
        "Ports": ([PortInfo], False),
    }


class HealthCheckConfig(AWSProperty):
    """
    `HealthCheckConfig <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lightsail-container-healthcheckconfig.html>`__
    """

    props: PropsDictType = {
        "HealthyThreshold": (integer, False),
        "IntervalSeconds": (integer, False),
        "Path": (str, False),
        "SuccessCodes": (str, False),
        "TimeoutSeconds": (integer, False),
        "UnhealthyThreshold": (integer, False),
    }


class PublicEndpoint(AWSProperty):
    """
    `PublicEndpoint <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lightsail-container-publicendpoint.html>`__
    """

    props: PropsDictType = {
        "ContainerName": (str, False),
        "ContainerPort": (integer, False),
        "HealthCheckConfig": (HealthCheckConfig, False),
    }


class ContainerServiceDeployment(AWSProperty):
    """
    `ContainerServiceDeployment <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lightsail-container-containerservicedeployment.html>`__
    """

    props: PropsDictType = {
        "Containers": ([ContainerProperty], False),
        "PublicEndpoint": (PublicEndpoint, False),
    }


class EcrImagePullerRole(AWSProperty):
    """
    `EcrImagePullerRole <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lightsail-container-ecrimagepullerrole.html>`__
    """

    props: PropsDictType = {
        "IsActive": (boolean, False),
        "PrincipalArn": (str, False),
    }


class PrivateRegistryAccess(AWSProperty):
    """
    `PrivateRegistryAccess <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lightsail-container-privateregistryaccess.html>`__
    """

    props: PropsDictType = {
        "EcrImagePullerRole": (EcrImagePullerRole, False),
    }


class PublicDomainName(AWSProperty):
    """
    `PublicDomainName <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lightsail-container-publicdomainname.html>`__
    """

    props: PropsDictType = {
        "CertificateName": (str, False),
        "DomainNames": ([str], False),
    }


class Container(AWSObject):
    """
    `Container <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lightsail-container.html>`__
    """

    resource_type = "AWS::Lightsail::Container"

    props: PropsDictType = {
        "ContainerServiceDeployment": (ContainerServiceDeployment, False),
        "IsDisabled": (boolean, False),
        "Power": (str, True),
        "PrivateRegistryAccess": (PrivateRegistryAccess, False),
        "PublicDomainNames": ([PublicDomainName], False),
        "Scale": (integer, True),
        "ServiceName": (str, True),
        "Tags": (Tags, False),
    }


class RelationalDatabaseParameter(AWSProperty):
    """
    `RelationalDatabaseParameter <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lightsail-database-relationaldatabaseparameter.html>`__
    """

    props: PropsDictType = {
        "AllowedValues": (str, False),
        "ApplyMethod": (str, False),
        "ApplyType": (str, False),
        "DataType": (str, False),
        "Description": (str, False),
        "IsModifiable": (boolean, False),
        "ParameterName": (str, False),
        "ParameterValue": (str, False),
    }


class Database(AWSObject):
    """
    `Database <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lightsail-database.html>`__
    """

    resource_type = "AWS::Lightsail::Database"

    props: PropsDictType = {
        "AvailabilityZone": (str, False),
        "BackupRetention": (boolean, False),
        "CaCertificateIdentifier": (str, False),
        "MasterDatabaseName": (str, True),
        "MasterUserPassword": (str, False),
        "MasterUsername": (str, True),
        "PreferredBackupWindow": (str, False),
        "PreferredMaintenanceWindow": (str, False),
        "PubliclyAccessible": (boolean, False),
        "RelationalDatabaseBlueprintId": (str, True),
        "RelationalDatabaseBundleId": (str, True),
        "RelationalDatabaseName": (str, True),
        "RelationalDatabaseParameters": ([RelationalDatabaseParameter], False),
        "RotateMasterUserPassword": (boolean, False),
        "Tags": (Tags, False),
    }


class AutoSnapshotAddOn(AWSProperty):
    """
    `AutoSnapshotAddOn <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lightsail-instance-autosnapshotaddon.html>`__
    """

    props: PropsDictType = {
        "SnapshotTimeOfDay": (str, False),
    }


class AddOn(AWSProperty):
    """
    `AddOn <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lightsail-instance-addon.html>`__
    """

    props: PropsDictType = {
        "AddOnType": (str, True),
        "AutoSnapshotAddOnRequest": (AutoSnapshotAddOn, False),
        "Status": (str, False),
    }


class Location(AWSProperty):
    """
    `Location <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lightsail-instance-location.html>`__
    """

    props: PropsDictType = {
        "AvailabilityZone": (str, False),
        "RegionName": (str, False),
    }


class Disk(AWSObject):
    """
    `Disk <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lightsail-disk.html>`__
    """

    resource_type = "AWS::Lightsail::Disk"

    props: PropsDictType = {
        "AddOns": ([AddOn], False),
        "AvailabilityZone": (str, False),
        "DiskName": (str, True),
        "Location": (Location, False),
        "SizeInGb": (integer, True),
        "Tags": (Tags, False),
    }


class CacheBehavior(AWSProperty):
    """
    `CacheBehavior <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lightsail-distribution-cachebehavior.html>`__
    """

    props: PropsDictType = {
        "Behavior": (str, False),
    }


class CacheBehaviorPerPath(AWSProperty):
    """
    `CacheBehaviorPerPath <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lightsail-distribution-cachebehaviorperpath.html>`__
    """

    props: PropsDictType = {
        "Behavior": (str, False),
        "Path": (str, False),
    }


class CookieObject(AWSProperty):
    """
    `CookieObject <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lightsail-distribution-cookieobject.html>`__
    """

    props: PropsDictType = {
        "CookiesAllowList": ([str], False),
        "Option": (str, False),
    }


class HeaderObject(AWSProperty):
    """
    `HeaderObject <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lightsail-distribution-headerobject.html>`__
    """

    props: PropsDictType = {
        "HeadersAllowList": ([str], False),
        "Option": (str, False),
    }


class QueryStringObject(AWSProperty):
    """
    `QueryStringObject <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lightsail-distribution-querystringobject.html>`__
    """

    props: PropsDictType = {
        "Option": (boolean, False),
        "QueryStringsAllowList": ([str], False),
    }


class CacheSettings(AWSProperty):
    """
    `CacheSettings <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lightsail-distribution-cachesettings.html>`__
    """

    props: PropsDictType = {
        "AllowedHTTPMethods": (str, False),
        "CachedHTTPMethods": (str, False),
        "DefaultTTL": (integer, False),
        "ForwardedCookies": (CookieObject, False),
        "ForwardedHeaders": (HeaderObject, False),
        "ForwardedQueryStrings": (QueryStringObject, False),
        "MaximumTTL": (integer, False),
        "MinimumTTL": (integer, False),
    }


class InputOrigin(AWSProperty):
    """
    `InputOrigin <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lightsail-distribution-inputorigin.html>`__
    """

    props: PropsDictType = {
        "Name": (str, False),
        "ProtocolPolicy": (str, False),
        "RegionName": (str, False),
    }


class Distribution(AWSObject):
    """
    `Distribution <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lightsail-distribution.html>`__
    """

    resource_type = "AWS::Lightsail::Distribution"

    props: PropsDictType = {
        "BundleId": (str, True),
        "CacheBehaviorSettings": (CacheSettings, False),
        "CacheBehaviors": ([CacheBehaviorPerPath], False),
        "CertificateName": (str, False),
        "DefaultCacheBehavior": (CacheBehavior, True),
        "DistributionName": (str, True),
        "IpAddressType": (str, False),
        "IsEnabled": (boolean, False),
        "Origin": (InputOrigin, True),
        "Tags": (Tags, False),
    }


class DiskProperty(AWSProperty):
    """
    `DiskProperty <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lightsail-instance-disk.html>`__
    """

    props: PropsDictType = {
        "AttachedTo": (str, False),
        "AttachmentState": (str, False),
        "DiskName": (str, True),
        "IOPS": (integer, False),
        "IsSystemDisk": (boolean, False),
        "Path": (str, True),
        "SizeInGb": (str, False),
    }


class Hardware(AWSProperty):
    """
    `Hardware <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lightsail-instance-hardware.html>`__
    """

    props: PropsDictType = {
        "CpuCount": (integer, False),
        "Disks": ([DiskProperty], False),
        "RamSizeInGb": (integer, False),
    }


class MonthlyTransfer(AWSProperty):
    """
    `MonthlyTransfer <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lightsail-instance-monthlytransfer.html>`__
    """

    props: PropsDictType = {
        "GbPerMonthAllocated": (str, False),
    }


class Port(AWSProperty):
    """
    `Port <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lightsail-instance-port.html>`__
    """

    props: PropsDictType = {
        "AccessDirection": (str, False),
        "AccessFrom": (str, False),
        "AccessType": (str, False),
        "CidrListAliases": ([str], False),
        "Cidrs": ([str], False),
        "CommonName": (str, False),
        "FromPort": (integer, False),
        "Ipv6Cidrs": ([str], False),
        "Protocol": (str, False),
        "ToPort": (integer, False),
    }


class Networking(AWSProperty):
    """
    `Networking <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lightsail-instance-networking.html>`__
    """

    props: PropsDictType = {
        "MonthlyTransfer": (MonthlyTransfer, False),
        "Ports": ([Port], True),
    }


class State(AWSProperty):
    """
    `State <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-lightsail-instance-state.html>`__
    """

    props: PropsDictType = {
        "Code": (integer, False),
        "Name": (str, False),
    }


class Instance(AWSObject):
    """
    `Instance <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lightsail-instance.html>`__
    """

    resource_type = "AWS::Lightsail::Instance"

    props: PropsDictType = {
        "AddOns": ([AddOn], False),
        "AvailabilityZone": (str, False),
        "BlueprintId": (str, True),
        "BundleId": (str, True),
        "Hardware": (Hardware, False),
        "InstanceName": (str, True),
        "KeyPairName": (str, False),
        "Location": (Location, False),
        "Networking": (Networking, False),
        "State": (State, False),
        "Tags": (Tags, False),
        "UserData": (str, False),
    }


class LoadBalancer(AWSObject):
    """
    `LoadBalancer <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lightsail-loadbalancer.html>`__
    """

    resource_type = "AWS::Lightsail::LoadBalancer"

    props: PropsDictType = {
        "AttachedInstances": ([str], False),
        "HealthCheckPath": (str, False),
        "InstancePort": (integer, True),
        "IpAddressType": (str, False),
        "LoadBalancerName": (str, True),
        "SessionStickinessEnabled": (boolean, False),
        "SessionStickinessLBCookieDurationSeconds": (str, False),
        "Tags": (Tags, False),
        "TlsPolicyName": (str, False),
    }


class LoadBalancerTlsCertificate(AWSObject):
    """
    `LoadBalancerTlsCertificate <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lightsail-loadbalancertlscertificate.html>`__
    """

    resource_type = "AWS::Lightsail::LoadBalancerTlsCertificate"

    props: PropsDictType = {
        "CertificateAlternativeNames": ([str], False),
        "CertificateDomainName": (str, True),
        "CertificateName": (str, True),
        "HttpsRedirectionEnabled": (boolean, False),
        "IsAttached": (boolean, False),
        "LoadBalancerName": (str, True),
    }


class StaticIp(AWSObject):
    """
    `StaticIp <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-lightsail-staticip.html>`__
    """

    resource_type = "AWS::Lightsail::StaticIp"

    props: PropsDictType = {
        "AttachedTo": (str, False),
        "StaticIpName": (str, True),
    }



import unittest

from troposphere.ssm import Document


class TestQueue(unittest.TestCase):
    def test_Document(self):
        # dict
        Document("title", Content={"foo": "bar"}).validate()
        # Valid yaml and json
        Document("title", Content='{"foo": "bar"}').validate()
        Document("title", Content="{foo: bar}").validate()
        # Invalid json/yaml
        with self.assertRaises(ValueError):
            Document("title", Content="*foo").validate()


if __name__ == "__main__":
    unittest.main()



import unittest

from troposphere import GetAtt, Join, Ref, Template
from troposphere.awslambda import (
    Code,
    Environment,
    Function,
    ImageConfig,
    validate_memory_size,
)
from troposphere.validators.awslambda import check_zip_file


class TestAWSLambda(unittest.TestCase):
    def test_exclusive(self):
        lambda_func = Function(
            "AMIIDLookup",
            Handler="index.handler",
            Role=GetAtt("LambdaExecutionRole", "Arn"),
            Code=Code(
                S3Bucket="lambda-functions",
                S3Key="amilookup.zip",
            ),
            Runtime="nodejs",
            Timeout="25",
        )
        t = Template()
        t.add_resource(lambda_func)
        t.to_json()

    def test_zip_file(self):
        lambda_func = Function(
            "AMIIDLookup",
            Handler="index.handler",
            Role=GetAtt("LambdaExecutionRole", "Arn"),
            Code=Code(
                ZipFile=Join(
                    "",
                    [
                        "var response = require('cfn-response');",
                        "exports.handler = function(event, context) {",
                        "  var input = parseInt(event.ResourceProperties.Input);",
                        "  var responseData = {Value: input * 5};",
                        "  response.send("
                        "    event, context, response.SUCCESS, responseData"
                        "  );",
                        "};",
                    ],
                ),
            ),
            Runtime="nodejs",
            Timeout="25",
        )
        t = Template()
        t.add_resource(lambda_func)
        t.to_json()

    def test_check_zip_file(self):
        four_mb = 4 * 1024 * 1024
        positive_tests = [
            "a" * four_mb,
            Join("", ["a" * four_mb]),
            Join("", ["a", 10]),
            Join("", ["a" * four_mb, Ref("EmptyParameter")]),
            GetAtt("foo", "bar"),
        ]
        for z in positive_tests:
            check_zip_file(z)
        negative_tests = [
            "a" * (four_mb + 1),
            Join("", ["a" * (four_mb + 1)]),
            Join("", ["a" * (four_mb + 1), Ref("EmptyParameter")]),
        ]
        for z in negative_tests:
            with self.assertRaises(ValueError):
                check_zip_file(z)

    def test_environment_variable_invalid_name(self):
        for var in ["1", "2var", "_var", "/var"]:
            with self.assertRaises(ValueError) as context:
                Environment(Variables={var: "value"})

            self.assertTrue(
                "Invalid environment variable name: %s" % var in context.exception.args
            )

    def test_environment_variable_reserved(self):
        for var in [
            "AWS_ACCESS_KEY",
            "AWS_ACCESS_KEY_ID",
            "AWS_LAMBDA_FUNCTION_MEMORY_SIZE",
        ]:
            with self.assertRaises(ValueError) as context:
                Environment(Variables={var: "value"})

            self.assertTrue(
                "Lambda Function environment variables names "
                "can't be none of" in context.exception.args[0]
            )

    def test_environment_variable_not_reserved(self):
        for var in ["NODE_PATH", "NODE_ENV", "FOO"]:
            try:
                Environment(Variables={var: "value"})
            except ValueError:
                self.fail("Environment() raised ValueError")

    def test_package_type_image(self):
        Function(
            "TestFunction",
            Code=Code(ImageUri="something"),
            PackageType="Image",
            Role=GetAtt("LambdaExecutionRole", "Arn"),
        ).validate()

    def test_package_type_invalid(self):
        with self.assertRaises(ValueError):
            Function(
                "TestFunction",
                Code=Code(ImageUri="something"),
                PackageType="Invalid",
                Role=GetAtt("LambdaExecutionRole", "Arn"),
            ).validate()

    def test_package_type_zip(self):
        Function(
            "TestFunction",
            Code=Code(
                ZipFile=Join(
                    "",
                    [
                        "var response = require('cfn-response');",
                        "exports.handler = function(event, context) {",
                        "  var input = parseInt(event.ResourceProperties.Input);",
                        "  var responseData = {Value: input * 5};",
                        "  response.send("
                        "    event, context, response.SUCCESS, responseData"
                        "  );",
                        "};",
                    ],
                ),
            ),
            Handler="index.handler",
            PackageType="Zip",
            Role=GetAtt("LambdaExecutionRole", "Arn"),
            Runtime="nodejs",
        ).validate()


class TestCode(unittest.TestCase):
    def test_validate_image_uri(self):
        Code(ImageUri="something").validate()

    def test_validate_image_and_zip(self):
        with self.assertRaises(ValueError):
            Code(ImageUri="something", ZipFile="something").validate()

    def test_validate_image_and_s3(self):
        s3_props = [
            {"S3Bucket": "bucket"},
            {"S3Key": "key"},
            {"S3ObjectVersion": "version"},
            {"S3Bucket": "bucket", "S3Key": "key"},
            {"S3Bucket": "bucket", "S3ObjectVersion": "version"},
            {"S3Key": "key", "S3ObjectVersion": "version"},
            {"S3Bucket": "bucket", "S3Key": "key", "S3ObjectVersion": "version"},
        ]
        for props in s3_props:
            with self.assertRaises(ValueError):
                Code(ImageUri="something", **props).validate()

    def test_validate_s3(self):
        Code(S3Bucket="bucket", S3Key="key").validate()
        Code(S3Bucket="bucket", S3Key="key", S3ObjectVersion="version").validate()

    def test_validate_s3_missing_required(self):
        s3_props = [
            {"S3Bucket": "bucket"},
            {"S3Key": "key"},
            {"S3ObjectVersion": "version"},
            {"S3Bucket": "bucket", "S3ObjectVersion": "version"},
            {"S3Key": "key", "S3ObjectVersion": "version"},
        ]
        for props in s3_props:
            with self.assertRaises(ValueError):
                Code(**props).validate()

    def test_validate_zip_and_s3(self):
        s3_props = [
            {"S3Bucket": "bucket"},
            {"S3Key": "key"},
            {"S3ObjectVersion": "version"},
            {"S3Bucket": "bucket", "S3Key": "key"},
            {"S3Bucket": "bucket", "S3ObjectVersion": "version"},
            {"S3Key": "key", "S3ObjectVersion": "version"},
            {"S3Bucket": "bucket", "S3Key": "key", "S3ObjectVersion": "version"},
        ]
        for props in s3_props:
            with self.assertRaises(ValueError):
                Code(ZipFile="something", **props).validate()


class TestImageConfig(unittest.TestCase):
    def test_validate_command(self):
        ImageConfig(Command=["something"] * 1500).validate()

    def test_validate_command_too_long(self):
        with self.assertRaises(ValueError):
            ImageConfig(Command=["something"] * 1501).validate()

    def test_validate_empty(self):
        ImageConfig().validate()

    def test_validate_entry_point(self):
        ImageConfig(EntryPoint=["something"] * 1500).validate()

    def test_validate_entry_point_too_long(self):
        with self.assertRaises(ValueError):
            ImageConfig(EntryPoint=["something"] * 1501).validate()

    def test_validate_working_directory(self):
        ImageConfig(WorkingDirectory="x" * 1000).validate()

    def test_validate_working_directory_too_long(self):
        with self.assertRaises(ValueError):
            ImageConfig(WorkingDirectory="x" * 1001).validate()

    def test_validate_memory_size_boundaries(self):
        for var in ["128", "129", "10240"]:
            validate_memory_size(var)

    def test_validate_memory_size_throws(self):
        for var in ["1", "111111111111111111111"]:
            with self.assertRaises(ValueError) as context:
                validate_memory_size(var)

            self.assertEqual(
                context.exception.args[0],
                "Lambda Function memory size must be between 128 and 10240",
            )


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2012-2022, Mark Peek <mark@peek.org>
# All rights reserved.
#
# See LICENSE file for full license.
#
# *** Do not modify - this file is autogenerated ***


from . import AWSObject, AWSProperty, PropsDictType, Tags
from .validators import boolean


class ApiGatewayProxyInput(AWSProperty):
    """
    `ApiGatewayProxyInput <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-refactorspaces-application-apigatewayproxyinput.html>`__
    """

    props: PropsDictType = {
        "EndpointType": (str, False),
        "StageName": (str, False),
    }


class Application(AWSObject):
    """
    `Application <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-refactorspaces-application.html>`__
    """

    resource_type = "AWS::RefactorSpaces::Application"

    props: PropsDictType = {
        "ApiGatewayProxy": (ApiGatewayProxyInput, False),
        "EnvironmentIdentifier": (str, True),
        "Name": (str, True),
        "ProxyType": (str, True),
        "Tags": (Tags, False),
        "VpcId": (str, True),
    }


class Environment(AWSObject):
    """
    `Environment <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-refactorspaces-environment.html>`__
    """

    resource_type = "AWS::RefactorSpaces::Environment"

    props: PropsDictType = {
        "Description": (str, False),
        "Name": (str, True),
        "NetworkFabricType": (str, True),
        "Tags": (Tags, False),
    }


class DefaultRouteInput(AWSProperty):
    """
    `DefaultRouteInput <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-refactorspaces-route-defaultrouteinput.html>`__
    """

    props: PropsDictType = {
        "ActivationState": (str, True),
    }


class UriPathRouteInput(AWSProperty):
    """
    `UriPathRouteInput <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-refactorspaces-route-uripathrouteinput.html>`__
    """

    props: PropsDictType = {
        "ActivationState": (str, True),
        "AppendSourcePath": (boolean, False),
        "IncludeChildPaths": (boolean, False),
        "Methods": ([str], False),
        "SourcePath": (str, False),
    }


class Route(AWSObject):
    """
    `Route <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-refactorspaces-route.html>`__
    """

    resource_type = "AWS::RefactorSpaces::Route"

    props: PropsDictType = {
        "ApplicationIdentifier": (str, True),
        "DefaultRoute": (DefaultRouteInput, False),
        "EnvironmentIdentifier": (str, True),
        "RouteType": (str, True),
        "ServiceIdentifier": (str, True),
        "Tags": (Tags, False),
        "UriPathRoute": (UriPathRouteInput, False),
    }


class LambdaEndpointInput(AWSProperty):
    """
    `LambdaEndpointInput <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-refactorspaces-service-lambdaendpointinput.html>`__
    """

    props: PropsDictType = {
        "Arn": (str, True),
    }


class UrlEndpointInput(AWSProperty):
    """
    `UrlEndpointInput <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-properties-refactorspaces-service-urlendpointinput.html>`__
    """

    props: PropsDictType = {
        "HealthUrl": (str, False),
        "Url": (str, True),
    }


class Service(AWSObject):
    """
    `Service <http://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-refactorspaces-service.html>`__
    """

    resource_type = "AWS::RefactorSpaces::Service"

    props: PropsDictType = {
        "ApplicationIdentifier": (str, True),
        "Description": (str, False),
        "EndpointType": (str, True),
        "EnvironmentIdentifier": (str, True),
        "LambdaEndpoint": (LambdaEndpointInput, False),
        "Name": (str, True),
        "Tags": (Tags, False),
        "UrlEndpoint": (UrlEndpointInput, False),
        "VpcId": (str, False),
    }
